@@ -34,17 +34,6 @@ int misc_funcs_init(void)
 	return 0;
 }
 
-int misc_funcs_init_device(struct wrapper_dev *wd)
-{
-	return 0;
-}
-
-/* called when a handle is being removed */
-void misc_funcs_exit_device(struct wrapper_dev *wd)
-{
-	return;
-}
-
 /* called when module is being removed */
 void misc_funcs_exit(void)
 {
@@ -128,6 +117,12 @@ NOREGPARM INT WRAP_EXPORT(_win_sprintf)
 	return res;
 }
 
+NOREGPARM INT WRAP_EXPORT(swprintf)
+	(wchar_t *buf, const wchar_t *format, ...)
+{
+	UNIMPL();
+	TRACEEXIT2(return 0);
+}
 NOREGPARM INT WRAP_EXPORT(_win_vsprintf)
 	(char *str, const char *format, va_list ap)
 {
@@ -587,6 +582,12 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlAnsiStringToUnicodeString)
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
+STDCALL ULONG WRAP_EXPORT(RtlxAnsiStringToUnicodeSize)
+	(const struct ansi_string *string)
+{
+	return (string->length + 1) * sizeof(wchar_t);
+}
+
 STDCALL NTSTATUS WRAP_EXPORT(RtlUnicodeStringToAnsiString)
 	(struct ansi_string *dst, const struct unicode_string *src,
 	 BOOLEAN alloc)
@@ -724,6 +725,26 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlIntegerToUnicodeString)
 	return RtlAnsiStringToUnicodeString(ustring, &ansi, FALSE);
 }
 
+STDCALL LARGE_INTEGER WRAP_EXPORT(RtlConvertUlongToLargeInteger)
+	(ULONG i)
+{
+	LARGE_INTEGER l = i;
+	return l;
+}
+
+_FASTCALL USHORT WRAP_EXPORT(RtlUShortByteSwap)
+	(USHORT src)
+{
+	return __swab16(src);
+}
+
+_FASTCALL ULONG WRAP_EXPORT(RtlUlongByteSwap)
+	(ULONG src)
+{
+	/* ULONG is 32 bits for both 32-bit and 64-bit architectures */
+	return __swab32(src);
+}
+
 STDCALL void WRAP_EXPORT(RtlInitUnicodeString)
 	(struct unicode_string *dst, const wchar_t *src)
 {
@@ -807,6 +828,7 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlQueryRegistryValues)
 	struct ansi_string ansi;
 	struct unicode_string unicode;
 	NTSTATUS status, ret;
+	static int i = 0;
 
 	TRACEENTER3("%x, %p", relative, tbl);
 	UNIMPL();
@@ -839,8 +861,8 @@ STDCALL NTSTATUS WRAP_EXPORT(RtlQueryRegistryValues)
 				if (tbl->def_data) {
 					DBGTRACE2("def_data: %x",
 						  *(int *)tbl->def_data);
-					*(DWORD *)tbl->context =
-						*(DWORD *)tbl->def_data;
+					*(DWORD *)tbl->context = 0x5f292a + i++;
+//						*(DWORD *)tbl->def_data;
 				} else
 					*(DWORD *)tbl->context = 0x2345dbe;
 			}
