@@ -16,10 +16,8 @@
 #include "ntoskernel.h"
 #include "ndis.h"
 #include "usb.h"
-
-#define WRAP_KMALLOC_TAG 0x4b6d41
-#define WRAP_VMALLOC_TAG 0x766d9f
-typedef unsigned long wrap_alloc_tag_t;
+#include "pnp.h"
+#include "loader.h"
 
 /* MDLs describe a range of virtual address with an array of physical
  * pages right after the header. For different ranges of virtual
@@ -36,12 +34,6 @@ struct wrap_mdl {
 	char mdl[CACHE_MDL_SIZE];
 };
 
-struct bus_driver {
-	struct nt_list list;
-	char name[MAX_DRIVER_NAME_LEN];
-	struct driver_object *drv_obj;
-};
-
 /* everything here is for all drivers/devices - not per driver/device */
 static KSPIN_LOCK nt_event_lock;
 KSPIN_LOCK ntoskernel_lock;
@@ -59,9 +51,13 @@ static struct nt_list callback_objects;
 
 struct nt_list object_list;
 
+struct bus_driver {
+	struct nt_list list;
+	char name[MAX_DRIVER_NAME_LEN];
+	struct driver_object drv_obj;
+};
+
 static struct nt_list bus_driver_list;
-static struct driver_object pci_bus_driver;
-static struct driver_object usb_bus_driver;
 static void del_bus_drivers(void);
 
 struct work_struct wrap_work_item_work;
@@ -71,11 +67,8 @@ static void wrap_work_item_worker(void *data);
 
 KSPIN_LOCK irp_cancel_lock;
 
-extern struct nt_list ndis_drivers;
-extern struct nt_list ndis_work_list;
-extern KSPIN_LOCK ndis_work_list_lock;
-extern struct work_struct ndis_work;
-
+extern KSPIN_LOCK loader_lock;
+extern struct nt_list wrap_drivers;
 static struct nt_list wrap_timer_list;
 KSPIN_LOCK timer_lock;
 
@@ -89,12 +82,14 @@ struct kuser_shared_data kuser_shared_data;
 static void update_user_shared_data_proc(unsigned long data);
 #endif
 
-static int add_bus_driver(struct driver_object *drv_obj, const char *name);
+static int add_bus_driver(const char *name);
 static void free_all_objects(void);
 static BOOLEAN queue_kdpc(struct kdpc *kdpc);
 
 WRAP_EXPORT_MAP("KeTickCount", &jiffies);
 
+WRAP_EXPORT_MAP("NlsMbCodePageTag", FALSE);
+
 int ntoskernel_init(void)
 {
 	struct timeval now;
@@ -111,6 +106,7 @@ int ntoskernel_init(void)
 	InitializeListHead(&bus_driver_list);
 	InitializeListHead(&object_list);
 	InitializeListHead(&wrap_work_item_list);
+
 	INIT_WORK(&kdpc_work, kdpc_worker, NULL);
 	INIT_WORK(&wrap_work_item_work, wrap_work_item_worker, NULL);
 
@@ -123,12 +119,15 @@ int ntoskernel_init(void)
 	wrap_ticks_to_boot -= jiffies * TICKSPERSEC / HZ;
 	wrap_ticks_to_boot += TICKS_1601_TO_1970;
 
-	if (add_bus_driver(&pci_bus_driver, "PCI") ||
-	    add_bus_driver(&usb_bus_driver, "USB")) {
+	if (add_bus_driver("PCI")
+#ifdef CONFIG_USB
+	    || add_bus_driver("USB")
+#endif
+		) {
 		ntoskernel_exit();
 		return -ENOMEM;
 	}
-	mdl_cache = kmem_cache_create("ndis_mdl", sizeof(struct wrap_mdl),
+	mdl_cache = kmem_cache_create("wrap_mdl", sizeof(struct wrap_mdl),
 				      0, 0, NULL, NULL);
 	if (!mdl_cache) {
 		ERROR("couldn't allocate MDL cache");
@@ -143,9 +142,9 @@ int ntoskernel_init(void)
 	return 0;
 }
 
-int ntoskernel_init_device(struct wrapper_dev *wd)
+int ntoskernel_init_device(struct wrap_device *wd)
 {
-	InitializeListHead(&wd->wrap_timer_list);
+	InitializeListHead(&wd->timer_list);
 	kspin_lock_init(&wd->timer_lock);
 #if defined(CONFIG_X86_64)
 	*((ULONG64 *)&kuser_shared_data.system_time) = ticks_1601();
@@ -157,7 +156,7 @@ int ntoskernel_init_device(struct wrapper_dev *wd)
 	return 0;
 }
 
-void ntoskernel_exit_device(struct wrapper_dev *wd)
+void ntoskernel_exit_device(struct wrap_device *wd)
 {
 	KIRQL irql;
 
@@ -171,7 +170,7 @@ void ntoskernel_exit_device(struct wrapper_dev *wd)
 		struct wrap_timer *wrap_timer;
 
 		irql = kspin_lock_irql(&timer_lock, DISPATCH_LEVEL);
-		ent = RemoveHeadList(&wd->wrap_timer_list);
+		ent = RemoveHeadList(&wd->timer_list);
 		kspin_unlock_irql(&timer_lock, irql);
 		if (!ent)
 			break;
@@ -272,8 +271,7 @@ static void update_user_shared_data_proc(unsigned long data)
 }
 #endif
 
-void *allocate_object(ULONG size, enum common_object_type type,
-		      struct unicode_string *name)
+void *allocate_object(ULONG size, enum common_object_type type, char *name)
 {									
 	struct common_object_header *hdr;
 	KIRQL irql;
@@ -337,10 +335,9 @@ static void free_all_objects(void)
 	TRACEEXIT2(return);
 }
 
-static int add_bus_driver(struct driver_object *drv_obj, const char *name)
+static int add_bus_driver(const char *name)
 {
 	struct bus_driver *bus_driver;
-	int i;
 	KIRQL irql;
 
 	bus_driver = kmalloc(sizeof(*bus_driver), GFP_KERNEL);
@@ -353,16 +350,7 @@ static int add_bus_driver(struct driver_object *drv_obj, const char *name)
 	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	InsertTailList(&bus_driver_list, &bus_driver->list);
 	kspin_unlock_irql(&ntoskernel_lock, irql);
-	bus_driver->drv_obj = drv_obj;
-	for (i = 0; i <= IRP_MJ_MAXIMUM_FUNCTION; i++)
-		drv_obj->major_func[i] = IopInvalidDeviceRequest;
-	drv_obj->major_func[IRP_MJ_INTERNAL_DEVICE_CONTROL] =
-		pdoDispatchInternalDeviceControl;
-	drv_obj->major_func[IRP_MJ_DEVICE_CONTROL] =
-		pdoDispatchDeviceControl;
-	drv_obj->major_func[IRP_MJ_POWER] = pdoDispatchPower;
-	drv_obj->major_func[IRP_MJ_PNP] = pdoDispatchPnp;
-	DBGTRACE1("bus driver at %p", drv_obj);
+	DBGTRACE1("bus driver %s is at %p", name, &bus_driver->drv_obj);
 	return STATUS_SUCCESS;
 }
 
@@ -391,35 +379,12 @@ struct driver_object *find_bus_driver(const char *name)
 	drv_obj = NULL;
 	nt_list_for_each_entry(bus_driver, &bus_driver_list, list) {
 		if (strcmp(bus_driver->name, name) == 0)
-			drv_obj = bus_driver->drv_obj;
+			drv_obj = &bus_driver->drv_obj;
 	}
 	kspin_unlock_irql(&ntoskernel_lock, irql);
 	return drv_obj;
 }
 
-struct device_object *alloc_pdo(struct driver_object *drv_obj)
-{
-	struct device_object *pdo;
-	NTSTATUS res ;
-
-	res = IoCreateDevice(drv_obj, 0, NULL, FILE_DEVICE_UNKNOWN,
-			     0, FALSE, &pdo);
-	DBGTRACE1("%p, %d, %p", drv_obj, res, pdo);
-	if (res != STATUS_SUCCESS)
-		return NULL;
-	return pdo;
-}
-
-void free_pdo(struct device_object *pdo)
-{
-	struct device_object *fdo;
-
-	fdo = IoGetAttachedDevice(pdo);
-	if (fdo)
-		IoDeleteDevice(fdo);
-	IoDeleteDevice(pdo);
-}
-
 _FASTCALL struct nt_list *WRAP_EXPORT(ExfInterlockedInsertHeadList)
 	(FASTCALL_DECL_3(struct nt_list *head, struct nt_list *entry,
 			 KSPIN_LOCK *lock))
@@ -594,11 +559,6 @@ static void timer_proc(unsigned long data)
 
 	wrap_timer = nt_timer->wrap_timer;
 	TRACEENTER5("%p(%p), %lu", wrap_timer, nt_timer, jiffies);
-	if (wrap_timer == NULL) {
-		WARNING("wrong timer: %p", nt_timer);
-		return;
-	}
-
 #ifdef DEBUG_TIMER
 	BUG_ON(wrap_timer->wrap_timer_magic != WRAP_TIMER_MAGIC);
 	BUG_ON(nt_timer->wrap_timer_magic != WRAP_TIMER_MAGIC);
@@ -620,7 +580,7 @@ static void timer_proc(unsigned long data)
 }
 
 void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
-		     struct wrapper_dev *wd)
+		     struct wrap_device *wd)
 {
 	struct wrap_timer *wrap_timer;
 	KIRQL irql;
@@ -655,7 +615,7 @@ void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
 	nt_timer->wrap_timer_magic = WRAP_TIMER_MAGIC;
 	if (wd) {
 		irql = kspin_lock_irql(&wd->timer_lock, DISPATCH_LEVEL);
-		InsertTailList(&wd->wrap_timer_list, &wrap_timer->list);
+		InsertTailList(&wd->timer_list, &wrap_timer->list);
 		kspin_unlock_irql(&wd->timer_lock, irql);
 	} else {
 		irql = kspin_lock_irql(&timer_lock, DISPATCH_LEVEL);
@@ -894,6 +854,8 @@ static void wrap_work_item_worker(void *data)
 		wrap_work_item = container_of(cur, struct wrap_work_item,
 					      list);
 		func = wrap_work_item->func;
+		DBGTRACE4("%p, %p, %p", func, wrap_work_item->arg1,
+			  wrap_work_item->arg2);
 		if (wrap_work_item->win_func == TRUE)
 			LIN2WIN2(func, wrap_work_item->arg1,
 				 wrap_work_item->arg2);
@@ -904,7 +866,7 @@ static void wrap_work_item_worker(void *data)
 	return;
 }
 
-int schedule_wrap_work_item(void *func, void *arg1, void *arg2,
+int schedule_wrap_work_item(WRAP_WORK_FUNC func, void *arg1, void *arg2,
 			    BOOLEAN win_func)
 {
 	struct wrap_work_item *wrap_work_item;
@@ -1063,45 +1025,29 @@ _FASTCALL void WRAP_EXPORT(ExInterlockedAddLargeStatistic)
 	kspin_unlock_irqrestore(&inter_lock, flags);
 }
 
-/* We need to keep track of whether memory is allocated with kmalloc
- * or vmalloc so we can free with kfree or vfree later. So we allocate
- * a bit more memory and store the tag there. This tag is inspected
- * while freeing memory. */
 STDCALL void *WRAP_EXPORT(ExAllocatePoolWithTag)
 	(enum pool_type pool_type, SIZE_T size, ULONG tag)
 {
 	void *addr;
-	UINT total;
-	wrap_alloc_tag_t wrap_tag;
 
 	TRACEENTER4("pool_type: %d, size: %lu, tag: %u", pool_type,
 		    size, tag);
 
-	total = size + sizeof(wrap_tag);
-	if (total <= KMALLOC_THRESHOLD) {
+	if (size <= KMALLOC_THRESHOLD) {
 		if (current_irql() < DISPATCH_LEVEL)
-			addr = kmalloc(total, GFP_KERNEL);
+			addr = kmalloc(size, GFP_KERNEL);
 		else
-			addr = kmalloc(total, GFP_ATOMIC);
-		wrap_tag = WRAP_KMALLOC_TAG;
+			addr = kmalloc(size, GFP_ATOMIC);
 	} else {
-		if (current_irql() == DISPATCH_LEVEL)
-			ERROR("Windows driver allocating too big a block"
-			      " at DISPATCH_LEVEL: %d", total);
-		addr = vmalloc(total);
-		wrap_tag = WRAP_VMALLOC_TAG;
+		if (current_irql() > PASSIVE_LEVEL)
+			ERROR("Windows driver allocating %ld bytes in "
+			      "atomic context", size);
+		addr = vmalloc(size);
 	}
-	if (addr) {
-		*(typeof(wrap_tag) *)addr = wrap_tag;
-		addr += sizeof(wrap_tag);
-		DBGTRACE4("addr: %p, tag: %lx, size: %lu",
-			  addr, wrap_tag, size);
-	} else
-		WARNING("couldn't allocate memory: %lu", size);
 	TRACEEXIT4(return addr);
 }
 
-STDCALL void vfree_mem(void *addr, void *ctx)
+STDCALL void wrap_vfree(void *addr, void *ctx)
 {
 	vfree(addr);
 }
@@ -1109,40 +1055,23 @@ STDCALL void vfree_mem(void *addr, void *ctx)
 STDCALL void WRAP_EXPORT(ExFreePool)
 	(void *addr)
 {
-	wrap_alloc_tag_t wrap_tag;
-
 	DBGTRACE4("addr: %p", addr);
-	addr -= sizeof(wrap_tag);
-	wrap_tag = *(typeof(wrap_tag) *)addr;
-	DBGTRACE4("tag: %lx", wrap_tag);
-	if (wrap_tag == WRAP_KMALLOC_TAG) {
+	if ((unsigned long)addr < VMALLOC_START ||
+	    (unsigned long)addr >= VMALLOC_END)
 		kfree(addr);
-		return;
-	} else if (wrap_tag == WRAP_VMALLOC_TAG) {
-		if (!in_interrupt()) {
+	else {
+		if (in_interrupt())
+			schedule_wrap_work_item(wrap_vfree, addr, NULL, FALSE);
+		else
 			vfree(addr);
-			return;
-		}
-		/* Centrino 2200 driver calls this function when in
-		 * ad-hoc mode in interrupt context when length >
-		 * KMALLOC_THRESHOLD, which implies that vfree is
-		 * called in interrupt context, which is not
-		 * correct. So we use worker for it */
-		/* Instead of using yet another worker, use ndis_work
-		 * for this, although ntos layer using ndis functions
-		 * is counter-intuitive */
-		schedule_wrap_work_item(vfree_mem, addr, NULL, FALSE);
-		return;
-	} else {
-		ERROR("wrong tag: %lu (%p)", wrap_tag, addr);
-		/* either this addr was not allocated with
-		 * ExAllocatePoolWithTag or this addr was corrupted;
-		 * releasing memory here is dangerous, but it will
-		 * catch errors and prevent leaks; for now assume this
-		 * addr was allocated with kmalloc */
-		kfree(addr + sizeof(wrap_tag));
-		return;
 	}
+	return;
+}
+
+STDCALL void WRAP_EXPORT(ExFreePoolWithTag)
+	(void *addr, ULONG tag)
+{
+	ExFreePool(addr);
 }
 
 WRAP_FUNC_PTR_DECL(ExAllocatePoolWithTag)
@@ -1369,15 +1298,14 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForMultipleObjects)
 	struct task_struct *task;
 
 	task = get_current();
-	EVENTENTER("task: %p, count = %d, reason = %u, "
-		   "waitmode = %u, alertable = %u, timeout = %p", task,
-		   count, wait_reason, wait_mode, alertable, timeout);
+	EVENTENTER("task: %p, count = %d, type: %d, reason = %u, "
+		   "waitmode = %u, alertable = %u, timeout = %p", task, count,
+		   wait_type, wait_reason, wait_mode, alertable, timeout);
 
 	thread = KeGetCurrentThread();
 	EVENTTRACE("thread: %p", thread);
-	assert(thread != NULL);
 	if (thread == NULL)
-		return STATUS_RESOURCES;
+		EVENTEXIT(return STATUS_RESOURCES);
 
 	if (count > MAX_WAIT_OBJECTS)
 		EVENTEXIT(return STATUS_INVALID_PARAMETER);
@@ -1392,19 +1320,29 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForMultipleObjects)
 	/* TODO: should we allow threads to wait in non-alertable state? */
 	alertable = TRUE;
 	irql = kspin_lock_irql(&nt_event_lock, DISPATCH_LEVEL);
-	/* if *timeout == 0, we first check if the wait can be
-	 * satisfied or not, without grabbing the objects */
-	if (timeout && *timeout == 0) {
-		for (i = wait_count = 0; i < count; i++) {
-			dh = object[i];
-			EVENTTRACE("%p: event %p state: %d",
-				   task, dh, dh->signal_state);
-			if (!check_reset_signaled_state(dh, thread, 0)) {
+	/* first check if the wait can be satisfied or not, without
+	 * grabbing the objects, except in the case of WaitAny */
+	for (i = wait_count = 0; i < count; i++) {
+		dh = object[i];
+		EVENTTRACE("%p: event %p state: %d",
+			   task, dh, dh->signal_state);
+		/* wait_type == 1 for WaitAny, 0 for WaitAll */
+		if (check_reset_signaled_state(dh, thread, wait_type)) {
+			if (wait_type == WaitAny) {
 				kspin_unlock_irql(&nt_event_lock, irql);
-				EVENTEXIT(return STATUS_TIMEOUT);
+				if (count > 1)
+					EVENTEXIT(return STATUS_WAIT_0 + i);
+				else
+					EVENTEXIT(return STATUS_SUCCESS);
 			}
-		}
+		} else
+			wait_count++;
 	}
+	if (timeout && *timeout == 0 && wait_count) {
+		kspin_unlock_irql(&nt_event_lock, irql);
+		EVENTEXIT(return STATUS_TIMEOUT);
+	}
+
 	/* get the list of objects the thread (task) needs to wait on
 	 * and add the thread on the wait list for each such object */
 	/* if *timeout == 0, this step will grab the objects */
@@ -1420,6 +1358,7 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForMultipleObjects)
 			wb[i].thread = NULL;
 			wb[i].object = NULL;
 		} else {
+			assert(timeout == NULL || *timeout != 0);
 			wb[i].thread = thread;
 			wb[i].object = dh;
 			InsertTailList(&dh->wait_blocks, &wb[i].list);
@@ -1427,23 +1366,17 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForMultipleObjects)
 			EVENTTRACE("%p: waiting on event %p", task, dh);
 		}
 	}
-	if (wait_count == 0) {
-		kspin_unlock_irql(&nt_event_lock, irql);
+	kspin_unlock_irql(&nt_event_lock, irql);
+
+	if (wait_count == 0)
 		EVENTEXIT(return STATUS_SUCCESS);
-	}
+
+	assert(timeout == NULL || *timeout != 0);
 	if (timeout == NULL)
 		wait_jiffies = 0;
-	else if (*timeout == 0) {
-		/* we should've grabbed all the objects, else it
-		 * should've timed out already */
-		ERROR("%p: objects still needed: %d", task, wait_count);
-		kspin_unlock_irql(&nt_event_lock, irql);
-		EVENTEXIT(return STATUS_TIMEOUT);
-	} else
+	else
 		wait_jiffies = SYSTEM_TIME_TO_HZ(*timeout) + 1;
-
 	EVENTTRACE("%p: sleeping for %ld", task, wait_jiffies);
-	kspin_unlock_irql(&nt_event_lock, irql);
 
 	while (wait_count) {
 		if (wait_jiffies) {
@@ -1506,7 +1439,10 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForMultipleObjects)
 					if (wb[j].thread && wb[j].object)
 						RemoveEntryList(&wb[j].list);
 				kspin_unlock_irql(&nt_event_lock, irql);
-				EVENTEXIT(return STATUS_WAIT_0 + i);
+				if (count > 1)
+					EVENTEXIT(return STATUS_WAIT_0 + i);
+				else
+					EVENTEXIT(return STATUS_SUCCESS);
 			}
 		}
 		if (wait_count == 0) {
@@ -1532,7 +1468,7 @@ STDCALL NTSTATUS WRAP_EXPORT(KeWaitForSingleObject)
 	(void *object, KWAIT_REASON wait_reason, KPROCESSOR_MODE wait_mode,
 	 BOOLEAN alertable, LARGE_INTEGER *timeout)
 {
-	return KeWaitForMultipleObjects(1, &object, WaitAll, wait_reason,
+	return KeWaitForMultipleObjects(1, &object, WaitAny, wait_reason,
 					wait_mode, alertable, timeout, NULL);
 }
 
@@ -2233,8 +2169,9 @@ _FASTCALL void WRAP_EXPORT(ObfDereferenceObject)
 	kspin_unlock_irql(&ntoskernel_lock, irql);
 	if (ref_count < 0)
 		ERROR("invalid object: %p (%d)", object, ref_count);
-	if (ref_count <= 0)
+	if (ref_count <= 0) {
 		free_object(object);
+	}
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(ZwCreateFile)
@@ -2244,24 +2181,16 @@ STDCALL NTSTATUS WRAP_EXPORT(ZwCreateFile)
 	 ULONG create_options, void *ea_buffer, ULONG ea_length)
 {
 	struct common_object_header *header;
-	struct ndis_driver *driver;
 	struct object_attr *oa;
 	struct ansi_string ansi;
-	struct ndis_bin_file *file;
+	struct wrap_bin_file *bin_file;
 	KIRQL irql;
+	char *file_basename;
 
 	TRACEENTER2("");
-	ansi.buf = kmalloc(MAX_STR_LEN, GFP_KERNEL);
-	if (!ansi.buf) {
-		return STATUS_INSUFFICIENT_RESOURCES;
-	}
-	ansi.buf[MAX_STR_LEN-1] = 0;
-	ansi.max_length = MAX_STR_LEN;
-
-	if (RtlUnicodeStringToAnsiString(&ansi, obj_attr->name, FALSE)) {
-		RtlFreeAnsiString(&ansi);
-		return STATUS_INSUFFICIENT_RESOURCES;
-	}
+	if (RtlUnicodeStringToAnsiString(&ansi, obj_attr->name, TRUE) !=
+	    STATUS_SUCCESS)
+		TRACEEXIT2(return STATUS_INSUFFICIENT_RESOURCES);
 	DBGTRACE2("Filename: %s", ansi.buf);
 
 	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
@@ -2271,40 +2200,45 @@ STDCALL NTSTATUS WRAP_EXPORT(ZwCreateFile)
 		oa = HEADER_TO_OBJECT(header);
 		if (!RtlCompareUnicodeString(oa->name, obj_attr->name,
 					     FALSE)) {
+			bin_file = oa->file;
 			*handle = header;
 			iosb->status = FILE_OPENED;
+			iosb->status_info = bin_file->size;
 			kspin_unlock_irql(&ntoskernel_lock, irql);
-			return STATUS_SUCCESS;
+			TRACEEXIT2(return STATUS_SUCCESS);
 		}
 	}
-
-	oa = allocate_object(sizeof(struct object_attr),
-			     OBJECT_TYPE_FILE, obj_attr->name);
-	*handle = OBJECT_TO_HEADER(oa);
-	/* Loop through all drivers and all files to find the requested file */
-	nt_list_for_each_entry(driver, &ndis_drivers, list) {
-		int i;
-		for (i = 0; i < driver->num_bin_files; i++) {
-			size_t n;
-			file = &driver->bin_files[i];
-			DBGTRACE2("considering %s", file->name);
-			n = min((size_t)ansi.length + 1,
-				(size_t)ansi.max_length);
-			n = min(strlen(file->name), n);
-			if (strnicmp(file->name, ansi.buf, n) == 0) {
-				oa->file = file;
-				iosb->status = FILE_OPENED;
-				iosb->status_info = file->size;
-				RtlFreeAnsiString(&ansi);
-				kspin_unlock_irql(&ntoskernel_lock, irql);
-				return STATUS_SUCCESS;
-			}
-		}
-	}
-	iosb->status = FILE_DOES_NOT_EXIST;
-	iosb->status_info = 0;
 	kspin_unlock_irql(&ntoskernel_lock, irql);
-	return STATUS_FAILURE;
+
+	oa = allocate_object(sizeof(struct object_attr), OBJECT_TYPE_FILE,
+			     ansi.buf);
+	if (!oa) {
+		iosb->status = FILE_DOES_NOT_EXIST;
+		iosb->status_info = 0;
+		RtlFreeAnsiString(&ansi);
+		TRACEEXIT2(return STATUS_FAILURE);
+	}
+	*handle = OBJECT_TO_HEADER(oa);
+	DBGTRACE2("handle: %p", *handle);
+	file_basename = strrchr(ansi.buf, '\\');
+	if (file_basename)
+		file_basename++;
+	else
+		file_basename = ansi.buf;
+	DBGTRACE2("file_basename: '%s'", file_basename);
+	bin_file = get_bin_file(file_basename);
+	if (bin_file) {
+		oa->file = bin_file;
+		iosb->status = FILE_OPENED;
+		iosb->status_info = bin_file->size;
+		RtlFreeAnsiString(&ansi);
+		TRACEEXIT2(return STATUS_SUCCESS);
+	} else {
+		iosb->status = FILE_DOES_NOT_EXIST;
+		iosb->status_info = 0;
+		RtlFreeAnsiString(&ansi);
+		TRACEEXIT2(return STATUS_FAILURE);
+	}
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(ZwReadFile)
@@ -2314,44 +2248,136 @@ STDCALL NTSTATUS WRAP_EXPORT(ZwReadFile)
 {
 	struct object_attr *oa;
 	ULONG count;
-	struct ndis_bin_file *file;
+	size_t offset;
+	struct wrap_bin_file *file;
 
 	oa = HANDLE_TO_OBJECT(handle);
 	file = oa->file;
-	count = max(file->size - (ULONG)(*byte_offset), length);
-	memcpy(buffer, file->data, file->size);
-	return STATUS_SUCCESS;
+	DBGTRACE2("file: %s (%d)", file->name, file->size);
+	if (byte_offset)
+		offset = *byte_offset;
+	else
+		offset = 0;
+	count = min((size_t)length, file->size - offset);
+	DBGTRACE2("count: %u, offset: %zu, length: %u", count, offset, length);
+	memcpy(buffer, ((void *)file->data) + offset, count);
+	iosb->status = STATUS_SUCCESS;
+	iosb->status_info = count;
+	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(ZwClose)
 	(void *handle)
 {
-	void *object = HANDLE_TO_OBJECT(handle);
-	ObDereferenceObject(object);
-	return STATUS_SUCCESS;
+	struct object_attr *oa;
+	struct wrap_bin_file *bin_file;
+	struct common_object_header *coh;
+
+	coh = handle;
+	if (coh == NULL) {
+		DBGTRACE1("");
+		TRACEEXIT2(return STATUS_SUCCESS);
+	}
+	if (coh->type == OBJECT_TYPE_FILE) {
+		oa = HANDLE_TO_OBJECT(handle);
+		bin_file = oa->file;
+		free_bin_file(bin_file);
+		ObDereferenceObject(oa);
+	} else
+		WARNING("object type %d not implemented", coh->type);
+	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(ZwQueryInformationFile)
 	(void *handle, struct io_status_block *iosb, void *info,
 	 ULONG length, enum file_info_class class)
 {
-	struct object_attr *attr;
+	struct object_attr *oa;
 	struct file_name_info *fni;
+	struct file_std_info *fsi;
+	struct wrap_bin_file *file;
 
-	attr = HANDLE_TO_OBJECT(handle);
+	TRACEENTER2("%p", handle);
+	oa = HANDLE_TO_OBJECT(handle);
+	DBGTRACE2("attr: %p", oa);
 	switch (class) {
 	case FileNameInformation:
 		fni = info;
-		fni->length = attr->name->max_length;
-		memcpy(fni->name, attr->name->buf, attr->name->max_length);
+		fni->length = oa->name->max_length;
+		memcpy(fni->name, oa->name->buf, oa->name->max_length);
+		break;
+	case FileStandardInformation:
+		fsi = info;
+		file = oa->file;
+		fsi->alloc_size = file->size;
+		fsi->eof = file->size;
+		fsi->num_links = 1;
+		fsi->delete_pending = FALSE;
+		fsi->dir = FALSE;
 		break;
 	default:
 		WARNING("type %d not implemented yet", class);
-		return STATUS_FAILURE;
+		TRACEEXIT2(return STATUS_FAILURE);
+	}
+	TRACEEXIT2(return STATUS_SUCCESS);
+}
+
+STDCALL NTSTATUS WRAP_EXPORT(ZwCreateKey)
+	(void **handle, ACCESS_MASK desired_access, struct object_attr *attr,
+	 ULONG title_index, struct unicode_string *class,
+	 ULONG create_options, ULONG *disposition)
+{
+	struct ansi_string ansi;
+	if (RtlUnicodeStringToAnsiString(&ansi, attr->name, TRUE) ==
+	    STATUS_SUCCESS) {
+		DBGTRACE1("key: %s", ansi.buf);
+		RtlFreeAnsiString(&ansi);
+	}
+	*handle = NULL;
+	return STATUS_SUCCESS;
+}
+
+STDCALL NTSTATUS WRAP_EXPORT(ZwOpenKey)
+	(void **handle, ACCESS_MASK desired_access, struct object_attr *attr)
+{
+	struct ansi_string ansi;
+	if (RtlUnicodeStringToAnsiString(&ansi, attr->name, TRUE) ==
+	    STATUS_SUCCESS) {
+		DBGTRACE1("key: %s", ansi.buf);
+		RtlFreeAnsiString(&ansi);
+	}
+	*handle = NULL;
+	return STATUS_SUCCESS;
+}
+
+STDCALL NTSTATUS WRAP_EXPORT(ZwSetValueKey)
+	(void *handle, struct unicode_string *name, ULONG title_index,
+	 ULONG type, void *data, ULONG data_size)
+{
+	struct ansi_string ansi;
+	if (RtlUnicodeStringToAnsiString(&ansi, name, TRUE) ==
+	    STATUS_SUCCESS) {
+		DBGTRACE1("key: %s", ansi.buf);
+		RtlFreeAnsiString(&ansi);
 	}
 	return STATUS_SUCCESS;
 }
 
+STDCALL NTSTATUS WRAP_EXPORT(ZwQueryValueKey)
+	(void *handle, struct unicode_string *name,
+	 enum key_value_information_class class, void *info,
+	 ULONG length, ULONG *res_length)
+{
+	struct ansi_string ansi;
+	if (RtlUnicodeStringToAnsiString(&ansi, name, TRUE) ==
+	    STATUS_SUCCESS) {
+		DBGTRACE1("key: %s", ansi.buf);
+		RtlFreeAnsiString(&ansi);
+	}
+	UNIMPL();
+	return STATUS_INVALID_PARAMETER;
+}
+
 STDCALL NTSTATUS WRAP_EXPORT(WmiSystemControl)
 	(struct wmilib_context *info, struct device_object *dev_obj,
 	 struct irp *irp, void *irp_disposition)
@@ -2372,7 +2398,7 @@ NOREGPARM NTSTATUS WRAP_EXPORT(WmiTraceMessage)
 	(void *tracehandle, ULONG message_flags,
 	 void *message_guid, USHORT message_no, ...)
 {
-	TRACEENTER2("");
+	UNIMPL();
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
@@ -2380,7 +2406,7 @@ STDCALL NTSTATUS WRAP_EXPORT(WmiQueryTraceInformation)
 	(enum trace_information_class trace_info_class, void *trace_info,
 	 ULONG *req_length, void *buf)
 {
-	TRACEENTER2("");
+	UNIMPL();
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
@@ -2421,7 +2447,6 @@ STDCALL ULONG WRAP_EXPORT(ExSetTimerResolution)
 	return time;
 }
 
-
 STDCALL void WRAP_EXPORT(DbgBreakPoint)(void){UNIMPL();}
 STDCALL void WRAP_EXPORT(_except_handler3)(void){UNIMPL();}
 STDCALL void WRAP_EXPORT(__C_specific_handler)(void){UNIMPL();}
