@@ -16,11 +16,13 @@
 #ifndef _NTOSKERNEL_H_
 #define _NTOSKERNEL_H_
 
-#define UTILS_VERSION "1.5"
+#define UTILS_VERSION "1.7"
 
 #include <linux/types.h>
 #include <linux/timer.h>
 #include <linux/time.h>
+#include <linux/module.h>
+#include <linux/kmod.h>
 
 #include <linux/netdevice.h>
 #include <linux/wireless.h>
@@ -73,33 +75,32 @@
 typedef struct workqueue_struct *workqueue;
 #include <asm/dma-mapping.h>
 
-#define PCI_DMA_ALLOC_COHERENT(pci_dev,size,dma_handle) \
-	dma_alloc_coherent(&pci_dev->dev,size,dma_handle, \
-			   GFP_KERNEL | __GFP_REPEAT | GFP_DMA)
-#define PCI_DMA_FREE_COHERENT(pci_dev,size,cpu_addr,dma_handle) \
+#define PCI_DMA_ALLOC_COHERENT(pci_dev,size,dma_handle)			\
+	dma_alloc_coherent(&pci_dev->dev,size,dma_handle, GFP_KERNEL)
+#define PCI_DMA_FREE_COHERENT(pci_dev,size,cpu_addr,dma_handle)		\
 	dma_free_coherent(&pci_dev->dev,size,cpu_addr,dma_handle)
-#define PCI_DMA_MAP_SINGLE(pci_dev,addr,size,direction) \
+#define PCI_DMA_MAP_SINGLE(pci_dev,addr,size,direction)		\
 	dma_map_single(&pci_dev->dev,addr,size,direction)
-#define PCI_DMA_UNMAP_SINGLE(pci_dev,dma_handle,size,direction) \
+#define PCI_DMA_UNMAP_SINGLE(pci_dev,dma_handle,size,direction)		\
 	dma_unmap_single(&pci_dev->dev,dma_handle,size,direction)
-#define MAP_SG(pci_dev, sglist, nents, direction) \
+#define MAP_SG(pci_dev, sglist, nents, direction)		\
 	dma_map_sg(&pci_dev->dev, sglist, nents, direction)
-#define UNMAP_SG(pci_dev, sglist, nents, direction) \
+#define UNMAP_SG(pci_dev, sglist, nents, direction)		\
 	dma_unmap_sg(&pci_dev->dev, sglist, nents, direction)
 
 #else // linux version <= 2.5.41
 
-#define PCI_DMA_ALLOC_COHERENT(dev,size,dma_handle) \
+#define PCI_DMA_ALLOC_COHERENT(dev,size,dma_handle)	\
 	pci_alloc_consistent(dev,size,dma_handle)
-#define PCI_DMA_FREE_COHERENT(dev,size,cpu_addr,dma_handle) \
+#define PCI_DMA_FREE_COHERENT(dev,size,cpu_addr,dma_handle)	\
 	pci_free_consistent(dev,size,cpu_addr,dma_handle)
-#define PCI_DMA_MAP_SINGLE(dev,addr,size,direction) \
+#define PCI_DMA_MAP_SINGLE(dev,addr,size,direction)	\
 	pci_map_single(dev,addr,size,direction)
-#define PCI_DMA_UNMAP_SINGLE(dev,dma_handle,size,direction) \
+#define PCI_DMA_UNMAP_SINGLE(dev,dma_handle,size,direction)	\
 	pci_unmap_single(dev,dma_handle,size,direction)
-#define MAP_SG(dev, sglist, nents, direction) \
+#define MAP_SG(dev, sglist, nents, direction)		\
 	pci_map_sg(dev, sglist, nents, direction)
-#define UNMAP_SG(dev, sglist, nents, direction) \
+#define UNMAP_SG(dev, sglist, nents, direction)		\
 	pci_unmap_sg(dev, sglist, nents, direction)
 #include <linux/tqueue.h>
 #define work_struct tq_struct
@@ -438,42 +439,114 @@ struct wrap_work_item {
 	BOOLEAN win_func;
 };
 
-typedef struct mdl ndis_buffer;
-
 #define MAX_ALLOCATED_URBS 15
 
-struct phys_dev {
-	int dev_type;
-	struct pci_dev *pci;
-	struct {
-		struct usb_device *udev;
-		struct usb_interface *intf;
-		int num_alloc_urbs;
-		struct nt_list wrap_urb_list;
-	} usb;
+struct wrap_device_setting {
+	struct nt_list list;
+	char name[MAX_SETTING_NAME_LEN];
+	char value[MAX_SETTING_VALUE_LEN];
+	void *encoded;
 };
 
-struct wrapper_dev;
+struct wrap_bin_file {
+	char driver_name[MAX_DRIVER_NAME_LEN];
+	char name[MAX_DRIVER_NAME_LEN];
+	int size;
+	void *data;
+};
+
+#define CE_WRAP_DRIVER_CLIENT_ID 1
+
+struct wrap_driver {
+	struct nt_list list;
+	struct driver_object *drv_obj;
+	char name[MAX_DRIVER_NAME_LEN];
+	char version[MAX_SETTING_VALUE_LEN];
+	unsigned int num_pe_images;
+	struct pe_image pe_images[MAX_DRIVER_PE_IMAGES];
+	int num_bin_files;
+	struct wrap_bin_file *bin_files;
+	struct nt_list wrap_devices;
+	union {
+		struct wrap_ndis_driver *ndis_driver;
+	};
+};
+
+enum hw_status {
+	HW_NORMAL, HW_SUSPENDED, HW_HALTED, HW_RMMOD, HW_AVAILABLE,
+	HW_INITIALIZED,
+};
+
+struct wrap_device {
+	struct nt_list list;
+	/* first part is (de)initialized once by loader */
+	int dev_bus_type;
+	int vendor;
+	int device;
+	int subvendor;
+	int subdevice;
+	struct wrap_driver *driver;
+	/* we need driver_name before driver is loaded */
+	char driver_name[MAX_DRIVER_NAME_LEN];
+	char conf_file_name[MAX_DRIVER_NAME_LEN];
+	struct nt_list settings;
+
+	/* rest should be (de)initialized during every
+	 * (de)initialization */
+	struct device_object *pdo;
+	union {
+		struct {
+			struct pci_dev *pdev;
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,9)
+			u32 pci_state[16];
+#endif
+		} pci;
+		struct {
+			struct usb_device *udev;
+			struct usb_interface *intf;
+			int num_alloc_urbs;
+			struct nt_list wrap_urb_list;
+		} usb;
+	};
+	union {
+		struct wrap_ndis_device *wnd;
+	};
+	struct nt_list timer_list;
+	KSPIN_LOCK timer_lock;
+	struct cm_resource_list *resource_list;
+};
 
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
 /* until issues with threads hogging cpu are resolved, we don't want
  * to use shared workqueue, lest the threads take keyboard etc down */
 #define USE_OWN_WORKQUEUE 1
-extern struct workqueue_struct *ndiswrapper_wq;
-#define schedule_work(work_struct) queue_work(ndiswrapper_wq, (work_struct))
+extern struct workqueue_struct *wrapper_wq;
+#define schedule_work(work_struct) queue_work(wrapper_wq, (work_struct))
 #endif
 
 int ntoskernel_init(void);
 void ntoskernel_exit(void);
-int ntoskernel_init_device(struct wrapper_dev *wd);
-void ntoskernel_exit_device(struct wrapper_dev *wd);
-void *allocate_object(ULONG size, enum common_object_type type,
-		      struct unicode_string *name);
+int ntoskernel_init_device(struct wrap_device *wd);
+void ntoskernel_exit_device(struct wrap_device *wd);
+void *allocate_object(ULONG size, enum common_object_type type, char *name);
 void  free_object(void *object);
 
+int usb_init(void);
+void usb_exit(void);
+int usb_init_device(struct wrap_device *wd);
+void usb_exit_device(struct wrap_device *wd);
+void usb_cancel_pending_urbs(void);
+
+int misc_funcs_init(void);
+void misc_funcs_exit(void);
+
+int wrap_procfs_init(void);
+void wrap_procfs_remove(void);
+
+int stricmp(const char *s1, const char *s2);
+void dump_bytes(const char *name, const u8 *from, int len);
+
 struct driver_object *find_bus_driver(const char *name);
-struct device_object *alloc_pdo(struct driver_object *drv_obj);
-void free_pdo(struct device_object *drv_obj);
 
 STDCALL void WRITE_PORT_UCHAR(ULONG_PTR port, UCHAR value);
 STDCALL UCHAR READ_PORT_UCHAR(ULONG_PTR port);
@@ -505,7 +578,6 @@ STDCALL struct mdl *IoAllocateMdl(void *virt, ULONG length, BOOLEAN second_buf,
 				  BOOLEAN charge_quota, struct irp *irp);
 STDCALL void MmBuildMdlForNonPagedPool(struct mdl *mdl);
 STDCALL void IoFreeMdl(struct mdl *mdl);
-STDCALL void NdisFreeBuffer(ndis_buffer *buffer);
 _FASTCALL LONG InterlockedDecrement(FASTCALL_DECL_1(LONG volatile *val));
 _FASTCALL LONG InterlockedIncrement(FASTCALL_DECL_1(LONG volatile *val));
 _FASTCALL struct nt_list *
@@ -539,19 +611,7 @@ STDCALL struct device_object *IoAttachDeviceToDeviceStack
 	(struct device_object *src, struct device_object *dst);
 STDCALL void KeInitializeEvent(struct nt_event *nt_event, enum event_type type,
 			       BOOLEAN state);
-void free_custom_ext(struct driver_extension *drv_obj_ext);
-STDCALL NTSTATUS AddDevice(struct driver_object *drv_obj,
-			   struct device_object *pdo);
-void DeleteDevice(struct device_object *pdo);
-
-driver_dispatch_t IopInvalidDeviceRequest;
-driver_dispatch_t IopPassIrpDown;
-driver_dispatch_t pdoDispatchInternalDeviceControl;
-driver_dispatch_t pdoDispatchDeviceControl;
-driver_dispatch_t pdoDispatchPnp;
-driver_dispatch_t pdoDispatchPower;
-driver_dispatch_t IopPassIrpDownAndWait;
-driver_dispatch_t fdoDispatchPnp;
+void free_custom_extensions(struct driver_extension *drv_obj_ext);
 
 STDCALL struct irp *IoAllocateIrp(char stack_size, BOOLEAN charge_quota);
 STDCALL void IoFreeIrp(struct irp *irp);
@@ -572,7 +632,7 @@ struct nt_thread *wrap_create_thread(struct task_struct *task);
 void wrap_remove_thread(struct nt_thread *thread);
 u64 ticks_1601(void);
 
-int schedule_wrap_work_item(void *func, void *arg1, void *arg2,
+int schedule_wrap_work_item(WRAP_WORK_FUNC func, void *arg1, void *arg2,
 			    BOOLEAN win_func);
 
 STDCALL KIRQL KeGetCurrentIrql(void);
@@ -616,7 +676,7 @@ NOREGPARM SIZE_T _win_wcslen(const wchar_t *s);
 void *wrap_kmalloc(size_t size);
 void wrap_kfree(void *ptr);
 void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
-		     struct wrapper_dev *wd);
+		     struct wrap_device *wd);
 BOOLEAN wrap_set_timer(struct nt_timer *nt_timer, unsigned long expires_hz,
 		       unsigned long repeat_hz, struct kdpc *kdpc);
 
@@ -883,7 +943,6 @@ extern int debug;
 #define DBGTRACE6(fmt, ...) DBGTRACE(6, fmt , ## __VA_ARGS__)
 #endif
 
-#define TRACEENTER(fmt, ...) DBGTRACE("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER1(fmt, ...) DBGTRACE1("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER2(fmt, ...) DBGTRACE2("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER3(fmt, ...) DBGTRACE3("Enter " fmt , ## __VA_ARGS__)
@@ -891,7 +950,6 @@ extern int debug;
 #define TRACEENTER5(fmt, ...) DBGTRACE5("Enter " fmt , ## __VA_ARGS__)
 #define TRACEENTER6(fmt, ...) DBGTRACE6("Enter " fmt , ## __VA_ARGS__)
 
-#define TRACEEXIT(stmt) do { DBGTRACE("Exit"); stmt; } while(0)
 #define TRACEEXIT1(stmt) do { DBGTRACE1("Exit"); stmt; } while(0)
 #define TRACEEXIT2(stmt) do { DBGTRACE2("Exit"); stmt; } while(0)
 #define TRACEEXIT3(stmt) do { DBGTRACE3("Exit"); stmt; } while(0)
