@@ -15,10 +15,12 @@
 
 #include "ntoskernel.h"
 #include "ndis.h"
-#include "wrapper.h"
+#include "wrapndis.h"
 #include "usb.h"
+#include "loader.h"
 
 extern KSPIN_LOCK ntoskernel_lock;
+extern KSPIN_LOCK loader_lock;
 extern KSPIN_LOCK irp_cancel_lock;
 extern struct nt_list object_list;
 
@@ -43,12 +45,9 @@ STDCALL NTSTATUS WRAP_EXPORT(IoGetDeviceProperty)
 {
 	struct ansi_string ansi;
 	struct unicode_string unicode;
-	struct wrapper_dev *wd;
 	char buf[32];
 	int devnum = 1;
 
-	wd = pdo->reserved;
-
 	IOENTER("dev_obj = %p, dev_property = %d, buffer_len = %u, "
 		"buffer = %p, result_len = %p", pdo, dev_property,
 		buffer_len, buffer, result_len);
@@ -87,7 +86,8 @@ STDCALL NTSTATUS WRAP_EXPORT(IoGetDeviceProperty)
 				IOEXIT(return STATUS_SUCCESS);
 			}
 		} else {
-			ansi.max_length = snprintf(buf, sizeof(buf), "%d", devnum);
+			ansi.max_length =
+				snprintf(buf, sizeof(buf), "%d", devnum);
 			*result_len = 2 * (ansi.max_length + 1);
 			IOEXIT(return STATUS_BUFFER_TOO_SMALL);
 		}
@@ -276,9 +276,11 @@ STDCALL struct irp *WRAP_EXPORT(IoBuildAsynchronousFsdRequest)
 	struct io_stack_location *irp_sl;
 
 	IOENTER("%p", dev_obj);
+	if (!dev_obj)
+		IOEXIT(return NULL);
 	irp = IoAllocateIrp(dev_obj->stack_size, FALSE);
 	if (irp == NULL)
-		return NULL;
+		IOEXIT(return NULL);
 
 	irp_sl = IoGetNextIrpStackLocation(irp);
 	irp_sl->major_fn = major_fn;
@@ -346,8 +348,9 @@ STDCALL struct irp *WRAP_EXPORT(IoBuildDeviceIoControlRequest)
 	struct irp *irp;
 	struct io_stack_location *irp_sl;
 
-	IOENTER("");
-
+	IOENTER("%p", dev_obj);
+	if (!dev_obj)
+		IOEXIT(return NULL);
 	irp = IoAllocateIrp(dev_obj->stack_size, FALSE);
 	if (irp) {
 		irp->user_status = io_status;
@@ -366,10 +369,10 @@ STDCALL struct irp *WRAP_EXPORT(IoBuildDeviceIoControlRequest)
 		irp_sl->flags = 0;
 		irp_sl->file_obj = NULL;
 		irp_sl->completion_routine = NULL;
+		IoQueueThreadIrp(irp);
 	}
 
 	IOTRACE("irp: %p", irp);
-	IoQueueThreadIrp(irp);
 	IOEXIT(return irp);
 }
 
@@ -382,7 +385,8 @@ _FASTCALL NTSTATUS WRAP_EXPORT(IofCallDriver)
 	struct driver_object *drv_obj;
 
 	DUMP_IRP(irp);
-
+	if (!dev_obj)
+		IOEXIT(return STATUS_SUCCESS);
 	drv_obj = dev_obj->drv_obj;
 	IOTRACE("drv_obj: %p", drv_obj);
 	IoSetNextIrpStackLocation(irp);
@@ -390,7 +394,7 @@ _FASTCALL NTSTATUS WRAP_EXPORT(IofCallDriver)
 
 	irp_sl->dev_obj = dev_obj;
 	major_func = drv_obj->major_func[irp_sl->major_fn];
-	IOTRACE("major_func: %p", major_func);
+	IOTRACE("major_func: %p, dev_obj: %p", major_func, dev_obj);
 	if (major_func)
 		res = (*major_func)(dev_obj, irp);
 	else {
@@ -413,7 +417,6 @@ _FASTCALL void WRAP_EXPORT(IofCompleteRequest)
 		ERROR("invalid irp: %p, STATUS_PENDING", irp);
 		return;
 	}
-
 	if (irp->current_location < 0) {
 		ERROR("invalid irp: %p, %d", irp, irp->current_location);
 		return;
@@ -476,211 +479,6 @@ _FASTCALL void WRAP_EXPORT(IofCompleteRequest)
 	IOEXIT(return);
 }
 
-STDCALL NTSTATUS
-pdoDispatchInternalDeviceControl(struct device_object *pdo,
-				 struct  irp *irp)
-{
-	struct io_stack_location *irp_sl;
-	NTSTATUS status;
-
-	DUMP_IRP(irp);
-
-	if (irp->current_location < 0 ||
-	    irp->current_location > irp->stack_count) {
-		ERROR("invalid irp: %p, %d, %d", irp, irp->current_location,
-		      irp->stack_count);
-		irp->io_status.status = STATUS_FAILURE;
-		irp->io_status.status_info = 0;
-		IOEXIT(return STATUS_FAILURE);
-	}
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-
-#ifdef CONFIG_USB
-	status = wrap_submit_irp(pdo, irp);
-	IOTRACE("status: %08X", status);
-	if (status == STATUS_PENDING) {
-		/* although as per DDK, we are not supposed to touch
-		 * irp when STAUS_PENDING is returned, this irp hasn't
-		 * been submitted to usb yet (and not completed), so
-		 * it is safe in this case */
-		status = wrap_submit_urb(irp);
-	}
-	if (status != STATUS_PENDING)
-		IoCompleteRequest(irp, IO_NO_INCREMENT);
-	IOEXIT(return status);
-#else
-	{
-		status = irp->io_status.status = STATUS_NOT_IMPLEMENTED;
-		IoCompleteRequest(irp, IO_NO_INCREMENT);
-		return status;
-	}
-#endif
-}
-
-STDCALL NTSTATUS pdoDispatchDeviceControl(struct device_object *pdo,
-					  struct irp *irp)
-{
-	return pdoDispatchInternalDeviceControl(pdo, irp);
-}
-
-STDCALL NTSTATUS IopInvalidDeviceRequest(struct device_object *dev_obj,
-					 struct irp *irp)
-{
-	struct io_stack_location *irp_sl;
-	NTSTATUS res;
-
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-	WARNING("IRP %d:%d not implemented",
-		irp_sl->major_fn, irp_sl->minor_fn);
-	irp->io_status.status = STATUS_NOT_IMPLEMENTED;
-	irp->io_status.status_info = 0;
-	res = irp->io_status.status;
-	IoCompleteRequest(irp, IO_NO_INCREMENT);
-	return res;
-}
-
-STDCALL NTSTATUS IopIrpWaitComplete(struct device_object *dev_obj,
-				    struct irp *irp, void *context)
-{
-	IOTRACE("dev_obj: %p, irp: %p, context: %p",
-		dev_obj, irp, context);
-	return STATUS_SUCCESS;
-}
-
-STDCALL NTSTATUS IopPassIrpDownAndWait(struct device_object *dev_obj,
-				       struct irp *irp)
-{
-	struct wrapper_dev *wd;
-	struct io_stack_location *irp_sl;
-	NTSTATUS status;
-
-	IOENTER("dev_obj: %p, irp: %p", dev_obj, irp);
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-	IoSetCompletionRoutine(irp, IopIrpWaitComplete, (void *)0x34b9f1,
-			       TRUE, TRUE, TRUE);
-	wd = dev_obj->reserved;
-	status = IoCallDriver(wd->nmb->pdo, irp);
-	return status;
-}
-
-STDCALL NTSTATUS IopPassIrpDown(struct device_object *dev_obj,
-				struct irp *irp)
-{
-	struct wrapper_dev *wd;
-	NTSTATUS status;
-
-	IOENTER("dev_obj: %p, irp: %p", dev_obj, irp);
-	IoSkipCurrentIrpStackLocation(irp);
-	wd = dev_obj->reserved;
-	status = IoCallDriver(wd->nmb->pdo, irp);
-	return status;
-}
-
-STDCALL NTSTATUS pdoDispatchPnp(struct device_object *pdo,
-				struct irp *irp)
-{
-	struct io_stack_location *irp_sl;
-	struct wrapper_dev *wd;
-	NTSTATUS res;
-
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-	wd = pdo->reserved;
-	IOTRACE("fn %d:%d, wd: %p", irp_sl->major_fn, irp_sl->minor_fn, wd);
-	switch (irp_sl->minor_fn) {
-	case IRP_MN_START_DEVICE:
-//		irp->io_status.status = miniport_init(wd);
-		break;
-	case IRP_MN_STOP_DEVICE:
-//		miniport_halt(wd);
-//		irp->io_status.status = STATUS_SUCCESS;
-		break;
-	case IRP_MN_QUERY_STOP_DEVICE:
-		irp->io_status.status = STATUS_SUCCESS;
-		break;
-	case IRP_MN_QUERY_REMOVE_DEVICE:
-		irp->io_status.status = STATUS_SUCCESS;
-		break;
-	case IRP_MN_REMOVE_DEVICE:
-//		miniport_halt(wd);
-//		free_pdo(wd->nmb->pdo);
-		irp->io_status.status = STATUS_SUCCESS;
-		break;
-	default:
-		WARNING("minor_fn: %d not implemented", irp_sl->minor_fn);
-		irp->io_status.status = STATUS_FAILURE;
-		break;
-	}
-	irp->io_status.status_info = 0;
-	res = irp->io_status.status;
-	IoCompleteRequest(irp, IO_NO_INCREMENT);
-	return res;
-}
-
-STDCALL NTSTATUS pdoDispatchPower(struct device_object *pdo,
-				  struct irp *irp)
-{
-	struct io_stack_location *irp_sl;
-	struct wrapper_dev *wd;
-	enum device_power_state state;
-	struct pci_dev *pdev;
-	NTSTATUS res;
-
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-	wd = pdo->reserved;
-	IOTRACE("pdo: %p, fn: %d:%d, wd: %p",
-		  pdo, irp_sl->major_fn, irp_sl->minor_fn, wd);
-	switch (irp_sl->minor_fn) {
-	case IRP_MN_SET_POWER:
-		state = irp_sl->params.power.state.device_state;
-		pdev = wd->dev.pci;
-		if (state == PowerDeviceD0) {
-			IOTRACE("resuming device %p", wd);
-			irp->io_status.status = STATUS_SUCCESS;
-		} else {
-			IOTRACE("suspending device %p", wd);
-			irp->io_status.status = STATUS_SUCCESS;
-		}
-		break;
-	case IRP_MN_QUERY_POWER:
-		irp->io_status.status = STATUS_SUCCESS;
-		break;
-	default:
-		irp->io_status.status = STATUS_SUCCESS;
-		ERROR("invalid power irp");
-		break;
-	}
-
-	irp->io_status.status_info = 0;
-	res = irp->io_status.status;
-	IoCompleteRequest(irp, IO_NO_INCREMENT);
-	return res;
-}
-
-STDCALL NTSTATUS fdoDispatchPnp(struct device_object *fdo,
-				struct irp *irp)
-{
-	struct io_stack_location *irp_sl;
-	struct wrapper_dev *wd;
-
-	IOENTER("fdo: %p, irp: %p", fdo, irp);
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-	IOTRACE("irp_sl: %p, handler: %p",
-		irp_sl, irp_sl->completion_routine);
-	wd = fdo->reserved;
-	switch (irp_sl->minor_fn) {
-	case IRP_MN_REMOVE_DEVICE:
-		/*
-		IoSkipCurrentIrpStackLocation(irp);
-		IoCallDriver(wd->nmb->pdo, irp);
-		free_pdo(wd->nmb->pdo);
-		*/
-		break;
-	case IRP_MN_START_DEVICE:
-		break;
-	}
-	return STATUS_SUCCESS;
-}
-
 static irqreturn_t io_irq_th(int irq, void *data, struct pt_regs *pt_regs)
 {
 	struct kinterrupt *interrupt = (struct kinterrupt *)data;
@@ -785,7 +583,6 @@ STDCALL struct io_workitem *WRAP_EXPORT(IoAllocateWorkItem)
 	io_workitem = kmalloc(sizeof(*io_workitem), GFP_ATOMIC);
 	if (!io_workitem)
 		IOEXIT(return NULL);
-
 	io_workitem->dev_obj = dev_obj;
 	IOEXIT(return io_workitem);
 }
@@ -810,10 +607,10 @@ STDCALL void WRAP_EXPORT(IoQueueWorkItem)
 	(struct io_workitem *io_workitem, void *func,
 	 enum work_queue_type queue_type, void *context)
 {
-	IOENTER("%p", io_workitem);
+	IOENTER("%p, %p", io_workitem, io_workitem->dev_obj);
 	io_workitem->worker_routine = func;
 	io_workitem->context = context;
-	schedule_wrap_work_item(func, io_workitem, context, TRUE);
+	schedule_wrap_work_item(func, io_workitem->dev_obj, context, TRUE);
 	IOEXIT(return);
 }
 
@@ -861,7 +658,7 @@ STDCALL void *WRAP_EXPORT(IoGetDriverObjectExtension)
 	IOEXIT(return ret);
 }
 
-void free_custom_ext(struct driver_extension *drv_ext)
+void free_custom_extensions(struct driver_extension *drv_ext)
 {
 	struct nt_list *ent;
 	KIRQL irql;
@@ -892,7 +689,7 @@ STDCALL NTSTATUS WRAP_EXPORT(IoCreateDevice)
 	}
 
 	size = sizeof(*dev) + dev_ext_length + sizeof(*dev_obj_ext);
-	dev = allocate_object(size, OBJECT_TYPE_DEVICE, dev_name);
+	dev = allocate_object(size, OBJECT_TYPE_DEVICE, ansi.buf);
 	if (!dev)
 		IOEXIT(return STATUS_INSUFFICIENT_RESOURCES);
 	if (dev_ext_length)
@@ -1017,7 +814,7 @@ STDCALL struct device_object *WRAP_EXPORT(IoGetAttachedDeviceReference)
 	if (!dev)
 		IOEXIT(return NULL);
 	d = IoGetAttachedDevice(dev);
-	ObReferenceObject(d);
+	d->ref_count++;
 	IOEXIT(return d);
 }
 
@@ -1036,6 +833,7 @@ STDCALL struct device_object *WRAP_EXPORT(IoAttachDeviceToDeviceStack)
 		dst->attached = src;
 	src->attached = NULL;
 	src->stack_size = dst->stack_size + 1;
+//	src->ref_count++;
 	kspin_unlock_irql(&ntoskernel_lock, irql);
 	IOTRACE("stack_size: %d -> %d", dst->stack_size, src->stack_size);
 	IOEXIT(return dst);
@@ -1053,25 +851,29 @@ STDCALL void WRAP_EXPORT(IoDetachDevice)
 	tail = topdev->attached;
 	if (!tail)
 		IOEXIT(return);
-	IOTRACE("tail:%p", tail);
+	IOTRACE("tail: %p", tail);
 
 	irql = kspin_lock_irql(&ntoskernel_lock, DISPATCH_LEVEL);
 	topdev->attached = tail->attached;
-	for (tail = topdev->attached; tail; tail = tail->attached)
+	IOTRACE("attached:%p", topdev->attached);
+	for ( ; tail; tail = tail->attached) {
+		IOTRACE("tail:%p", tail);
 		tail->stack_size--;
+	}
 	kspin_unlock_irql(&ntoskernel_lock, irql);
-	ObDereferenceObject(topdev);
 	IOEXIT(return);
 }
 
-STDCALL union power_state WRAP_EXPORT(PoSetPowerState)
+/* gcc will return unions and structs either through registers or in
+ * memory, so we can't return 'union power_state'; instead, we return
+ * 'enum device_power_state' (which is one of the fields in the
+ * union). With this gcc will honor STDCALL and return value
+ * appropriately */
+STDCALL enum device_power_state WRAP_EXPORT(PoSetPowerState)
 	(struct device_object *dev_obj, enum power_state_type type,
 	 union power_state state)
 {
-	union power_state ps;
-
-	ps.device_state = PowerDeviceD0;
-	return ps;
+	IOEXIT(return PowerDeviceD0);
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(PoCallDriver)
@@ -1086,21 +888,22 @@ STDCALL NTSTATUS WRAP_EXPORT(PoRequestPowerIrp)
 	 void *context, struct irp **pirp)
 {
 	struct irp *irp;
-	struct io_stack_location *stack;
+	struct io_stack_location *irp_sl;
 
 	DBGTRACE1("%p: stack size: %d", dev_obj, dev_obj->stack_size);
 	DBGTRACE1("drv_obj: %p", dev_obj->drv_obj);
 	irp = IoAllocateIrp(dev_obj->stack_size, FALSE);
 	if (!irp)
 		return STATUS_INSUFFICIENT_RESOURCES;
-	stack = IoGetNextIrpStackLocation(irp);
-	stack->major_fn = IRP_MJ_POWER;
-	stack->minor_fn = minor_fn;
+	irp_sl = IoGetNextIrpStackLocation(irp);
+	irp_sl->major_fn = IRP_MJ_POWER;
+	irp_sl->minor_fn = minor_fn;
 	if (minor_fn == IRP_MN_WAIT_WAKE)
-		stack->params.power.type = SystemPowerState;
+		irp_sl->params.power.type = SystemPowerState;
 	else
-		stack->params.power.type = DevicePowerState;
-	stack->params.power.state = power_state;
+		irp_sl->params.power.type = DevicePowerState;
+	irp_sl->params.power.state = power_state;
+	irp_sl->completion_routine = completion_func;
 	irp->io_status.status = STATUS_NOT_SUPPORTED;
 	*pirp = irp;
 	return PoCallDriver(dev_obj, irp);
@@ -1113,6 +916,29 @@ STDCALL void WRAP_EXPORT(PoStartNextPowerIrp)
 	IOEXIT(return);
 }
 
+STDCALL void WRAP_EXPORT(IoInitializeRemoveLockEx)
+	(struct io_remove_lock *lock, ULONG alloc_tag, ULONG max_locked_min,
+	 ULONG high_mark, ULONG lock_size)
+{
+	UNIMPL();
+}
+
+
+STDCALL NTSTATUS WRAP_EXPORT(IoAcquireRemoveLockEx)
+	(struct io_remove_lock lock, void *tag, char *file, ULONG line,
+	 ULONG lock_size)
+{
+	UNIMPL();
+	IOEXIT(return STATUS_SUCCESS);
+}
+
+STDCALL NTSTATUS WRAP_EXPORT(IoReleaseRemoveLockEx)
+	(struct io_remove_lock lock, void *tag, ULONG lock_size)
+{
+	UNIMPL();
+	IOEXIT(return STATUS_SUCCESS);
+}
+
 STDCALL NTSTATUS WRAP_EXPORT(IoRegisterDeviceInterface)
 	(struct device_object *pdo, struct guid *guid_class,
 	 struct unicode_string *reference, struct unicode_string *link)
@@ -1135,39 +961,14 @@ STDCALL NTSTATUS WRAP_EXPORT(IoSetDeviceInterfaceState)
 }
 
 STDCALL NTSTATUS WRAP_EXPORT(IoOpenDeviceRegistryKey)
-	(struct device_object *pdo, ULONG type, ACCESS_MASK mask,
+	(struct device_object *dev_obj, ULONG type, ACCESS_MASK mask,
 	 void **handle)
 {
-	struct wrapper_dev *wd;
-
-	TRACEENTER1("pdo: %p", pdo);
-	wd = pdo->reserved;
-	*handle = wd->nmb;
+	TRACEENTER1("dev_obj: %p", dev_obj);
+	*handle = dev_obj;
 	return STATUS_SUCCESS;
 }
 
-STDCALL NTSTATUS WRAP_EXPORT(ZwQueryValueKey)
-	(void *handle, struct unicode_string *name,
-	 enum key_value_information_class class, void *info,
-	 ULONG length, ULONG *res_length)
-{
-	NDIS_STATUS status;
-	struct ndis_config_param *param;
-
-	NdisReadConfiguration(&status, &param, handle, name,
-			      NDIS_CONFIG_PARAM_STRING);
-	if (status == NDIS_STATUS_SUCCESS) {
-		*res_length = param->data.ustring.length;
-		if (length < param->data.ustring.length) {
-			RtlCopyMemory(info, param->data.ustring.buf,
-				      *res_length);
-			return STATUS_SUCCESS;
-		} else
-			return STATUS_BUFFER_TOO_SMALL;
-	} else
-		return STATUS_INVALID_PARAMETER;
-}
-
 STDCALL unsigned int WRAP_EXPORT(IoWMIRegistrationControl)
 	(struct device_object *dev_obj, ULONG action)
 {
@@ -1214,4 +1015,18 @@ STDCALL unsigned int WRAP_EXPORT(IoWMIRegistrationControl)
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
+STDCALL void WRAP_EXPORT(IoInvalidateDeviceRelations)
+	(struct device_object *dev_obj, enum device_relation_type type)
+{
+	INFO("%p, %d", dev_obj, type);
+	UNIMPL();
+}
+
+STDCALL void WRAP_EXPORT(IoInvalidateDeviceState)
+	(struct device_object *pdo)
+{
+	INFO("%p", pdo);
+	UNIMPL();
+}
+
 #include "ntoskernel_io_exports.h"
