@@ -18,18 +18,19 @@
 
 #include "ndis.h"
 #include "iw_ndis.h"
-#include "wrapper.h"
+#include "wrapndis.h"
+#include "pnp.h"
 
 #define MAX_PROC_STR_LEN 32
 
-static struct proc_dir_entry *ndiswrapper_procfs_entry;
+static struct proc_dir_entry *wrap_procfs_entry;
 extern int proc_uid, proc_gid;
 
-static int procfs_read_stats(char *page, char **start, off_t off,
-			     int count, int *eof, void *data)
+static int procfs_read_ndis_stats(char *page, char **start, off_t off,
+				  int count, int *eof, void *data)
 {
 	char *p = page;
-	struct wrapper_dev *wd = (struct wrapper_dev *)data;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	struct ndis_wireless_stats stats;
 	NDIS_STATUS res;
 	ndis_rssi rssi;
@@ -39,11 +40,11 @@ static int procfs_read_stats(char *page, char **start, off_t off,
 		return 0;
 	}
 
-	res = miniport_query_info(wd, OID_802_11_RSSI, &rssi, sizeof(rssi));
+	res = miniport_query_info(wnd, OID_802_11_RSSI, &rssi, sizeof(rssi));
 	if (!res)
 		p += sprintf(p, "signal_level=%d dBm\n", (s32)rssi);
 
-	res = miniport_query_info(wd, OID_802_11_STATISTICS,
+	res = miniport_query_info(wnd, OID_802_11_STATISTICS,
 				  &stats, sizeof(stats));
 	if (!res) {
 
@@ -72,11 +73,11 @@ static int procfs_read_stats(char *page, char **start, off_t off,
 	return (p - page);
 }
 
-static int procfs_read_encr(char *page, char **start, off_t off,
-			    int count, int *eof, void *data)
+static int procfs_read_ndis_encr(char *page, char **start, off_t off,
+				 int count, int *eof, void *data)
 {
 	char *p = page;
-	struct wrapper_dev *wd = (struct wrapper_dev *)data;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	int i, encr_status, auth_mode, infra_mode;
 	NDIS_STATUS res;
 	struct ndis_essid essid;
@@ -87,7 +88,7 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 		return 0;
 	}
 
-	res = miniport_query_info(wd, OID_802_11_BSSID,
+	res = miniport_query_info(wnd, OID_802_11_BSSID,
 				  &ap_address, sizeof(ap_address));
 	if (res)
 		memset(ap_address, 0, ETH_ALEN);
@@ -96,28 +97,28 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 		p += sprintf(p, ":%2.2X", ap_address[i]);
 	p += sprintf(p, "\n");
 
-	res = miniport_query_info(wd, OID_802_11_SSID, &essid,
+	res = miniport_query_info(wnd, OID_802_11_SSID, &essid,
 				  sizeof(essid));
 	if (!res) {
 		essid.essid[essid.length] = '\0';
 		p += sprintf(p, "essid=%s\n", essid.essid);
 	}
 
-	res = miniport_query_int(wd, OID_802_11_ENCRYPTION_STATUS,
+	res = miniport_query_int(wnd, OID_802_11_ENCRYPTION_STATUS,
 				 &encr_status);
-	res |= miniport_query_int(wd, OID_802_11_AUTHENTICATION_MODE,
+	res |= miniport_query_int(wnd, OID_802_11_AUTHENTICATION_MODE,
 				  &auth_mode);
 
 	if (!res) {
-		int t = wd->encr_info.tx_key_index;
-		p += sprintf(p, "tx_key=%u\n", wd->encr_info.tx_key_index);
+		int t = wnd->encr_info.tx_key_index;
+		p += sprintf(p, "tx_key=%u\n", wnd->encr_info.tx_key_index);
 		p += sprintf(p, "key=");
-		if (wd->encr_info.keys[t].length > 0)
+		if (wnd->encr_info.keys[t].length > 0)
 			for (i = 0; i < NDIS_ENCODING_TOKEN_MAX &&
-				     i < wd->encr_info.keys[t].length;
+				     i < wnd->encr_info.keys[t].length;
 			     i++)
 				p += sprintf(p, "%2.2X",
-					     wd->encr_info.keys[t].key[i]);
+					     wnd->encr_info.keys[t].key[i]);
 		else
 			p += sprintf(p, "off");
 		p += sprintf(p, "\n");
@@ -126,7 +127,7 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 		p += sprintf(p, "auth_mode=%d\n", auth_mode);
 	}
 
-	res = miniport_query_int(wd, OID_802_11_INFRASTRUCTURE_MODE,
+	res = miniport_query_int(wnd, OID_802_11_INFRASTRUCTURE_MODE,
 				 &infra_mode);
 	p += sprintf(p, "mode=%s\n", (infra_mode == Ndis802_11IBSS) ?
 		     "adhoc" : (infra_mode == Ndis802_11Infrastructure) ?
@@ -140,11 +141,11 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 	return (p - page);
 }
 
-static int procfs_read_hw(char *page, char **start, off_t off,
-			  int count, int *eof, void *data)
+static int procfs_read_ndis_hw(char *page, char **start, off_t off,
+			       int count, int *eof, void *data)
 {
 	char *p = page;
-	struct wrapper_dev *wd = (struct wrapper_dev *)data;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	struct ndis_configuration config;
 	unsigned int power_mode;
 	NDIS_STATUS res;
@@ -159,7 +160,7 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 		return 0;
 	}
 
-	res = miniport_query_info(wd, OID_802_11_CONFIGURATION,
+	res = miniport_query_info(wnd, OID_802_11_CONFIGURATION,
 				  &config, sizeof(config));
 	if (!res) {
 		p += sprintf(p, "beacon_period=%u msec\n",
@@ -174,27 +175,27 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 			     config.fh_config.dwell_time);
 	}
 
-	res = miniport_query_info(wd, OID_802_11_TX_POWER_LEVEL,
+	res = miniport_query_info(wnd, OID_802_11_TX_POWER_LEVEL,
 				  &tx_power, sizeof(tx_power));
 	if (!res)
 		p += sprintf(p, "tx_power=%u mW\n", tx_power);
 
-	res = miniport_query_info(wd, OID_GEN_LINK_SPEED,
+	res = miniport_query_info(wnd, OID_GEN_LINK_SPEED,
 				  &bit_rate, sizeof(bit_rate));
 	if (!res)
 		p += sprintf(p, "bit_rate=%u kBps\n", (u32)bit_rate / 10);
 
-	res = miniport_query_info(wd, OID_802_11_RTS_THRESHOLD,
+	res = miniport_query_info(wnd, OID_802_11_RTS_THRESHOLD,
 				  &rts_threshold, sizeof(rts_threshold));
 	if (!res)
 		p += sprintf(p, "rts_threshold=%u bytes\n", rts_threshold);
 
-	res = miniport_query_info(wd, OID_802_11_FRAGMENTATION_THRESHOLD,
+	res = miniport_query_info(wnd, OID_802_11_FRAGMENTATION_THRESHOLD,
 				  &frag_threshold, sizeof(frag_threshold));
 	if (!res)
 		p += sprintf(p, "frag_threshold=%u bytes\n", frag_threshold);
 
-	res = miniport_query_int(wd, OID_802_11_POWER_MODE, &power_mode);
+	res = miniport_query_int(wnd, OID_802_11_POWER_MODE, &power_mode);
 	if (!res)
 		p += sprintf(p, "power_mode=%s\n",
 			     (power_mode == NDIS_POWER_OFF) ?
@@ -202,37 +203,37 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 			     (power_mode == NDIS_POWER_MAX) ?
 			     "max_savings" : "min_savings");
 
-	res = miniport_query_info(wd, OID_802_11_NUMBER_OF_ANTENNAS,
+	res = miniport_query_info(wnd, OID_802_11_NUMBER_OF_ANTENNAS,
 				  &antenna, sizeof(antenna));
 	if (!res)
 		p += sprintf(p, "num_antennas=%u\n", antenna);
 
-	res = miniport_query_info(wd, OID_802_11_TX_ANTENNA_SELECTED,
+	res = miniport_query_info(wnd, OID_802_11_TX_ANTENNA_SELECTED,
 				  &antenna, sizeof(antenna));
 	if (!res)
 		p += sprintf(p, "tx_antenna=%u\n", antenna);
 
-	res = miniport_query_info(wd, OID_802_11_RX_ANTENNA_SELECTED,
+	res = miniport_query_info(wnd, OID_802_11_RX_ANTENNA_SELECTED,
 				  &antenna, sizeof(antenna));
 	if (!res)
 		p += sprintf(p, "rx_antenna=%u\n", antenna);
 
 	p += sprintf(p, "encryption_modes=%s%s%s%s%s%s%s\n",
-		     test_bit(Ndis802_11Encryption1Enabled, &wd->capa.encr) ?
+		     test_bit(Ndis802_11Encryption1Enabled, &wnd->capa.encr) ?
 		     "WEP" : "none",
 
-		     test_bit(Ndis802_11Encryption2Enabled, &wd->capa.encr) ?
+		     test_bit(Ndis802_11Encryption2Enabled, &wnd->capa.encr) ?
 		     "; TKIP with WPA" : "",
-		     test_bit(Ndis802_11AuthModeWPA2, &wd->capa.auth) ?
+		     test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		     ", WPA2" : "",
-		     test_bit(Ndis802_11AuthModeWPA2PSK, &wd->capa.auth) ?
+		     test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
 		     ", WPA2PSK" : "",
 
-		     test_bit(Ndis802_11Encryption3Enabled, &wd->capa.encr) ?
+		     test_bit(Ndis802_11Encryption3Enabled, &wnd->capa.encr) ?
 		     "; AES/CCMP with WPA" : "",
-		     test_bit(Ndis802_11AuthModeWPA2, &wd->capa.auth) ?
+		     test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		     ", WPA2" : "",
-		     test_bit(Ndis802_11AuthModeWPA2PSK, &wd->capa.auth) ?
+		     test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
 		     ", WPA2PSK" : "");
 
 	if (p - page > count) {
@@ -244,12 +245,12 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 	return (p - page);
 }
 
-static int procfs_read_settings(char *page, char **start, off_t off,
-				int count, int *eof, void *data)
+static int procfs_read_ndis_settings(char *page, char **start, off_t off,
+				     int count, int *eof, void *data)
 {
 	char *p = page;
-	struct wrapper_dev *wd = (struct wrapper_dev *)data;
-	struct device_setting *setting;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
+	struct wrap_device_setting *setting;
 
 	if (off != 0) {
 		*eof = 1;
@@ -257,19 +258,19 @@ static int procfs_read_settings(char *page, char **start, off_t off,
 	}
 
 	p += sprintf(p, "hangcheck_interval=%d\n",
-		     (int)(wd->hangcheck_interval / HZ));
+		     (int)(wnd->hangcheck_interval / HZ));
 
-	list_for_each_entry(setting, &wd->ndis_device->settings, list) {
+	list_for_each_entry(setting, &wnd->wd->settings, list) {
 		p += sprintf(p, "%s=%s\n", setting->name, setting->value);
 	}
 
 	return (p - page);
 }
 
-static int procfs_write_settings(struct file *file, const char *buf,
-				 unsigned long count, void *data)
+static int procfs_write_ndis_settings(struct file *file, const char *buf,
+				      unsigned long count, void *data)
 {
-	struct wrapper_dev *wd = (struct wrapper_dev *)data;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	char setting[MAX_PROC_STR_LEN], *p;
 
 	if (count > MAX_PROC_STR_LEN)
@@ -292,28 +293,32 @@ static int procfs_write_settings(struct file *file, const char *buf,
 			return -EINVAL;
 		p++;
 		i = simple_strtol(p, NULL, 10);
-		hangcheck_del(wd);
-		wd->hangcheck_interval = i * HZ;
-		hangcheck_add(wd);
+		hangcheck_del(wnd);
+		wnd->hangcheck_interval = i * HZ;
+		hangcheck_add(wnd);
 	} else if (!strcmp(setting, "check_capa")) {
-		check_capa(wd);
-		printk(KERN_INFO "%s: encryption modes supported: %s%s%s%s%s%s%s\n",
-		       wd->net_dev->name,
-		       test_bit(Ndis802_11Encryption1Enabled, &wd->capa.encr) ?
+		check_capa(wnd);
+		printk(KERN_INFO
+		       "%s: encryption modes supported: %s%s%s%s%s%s%s\n",
+		       wnd->net_dev->name,
+		       test_bit(Ndis802_11Encryption1Enabled,
+				&wnd->capa.encr) ?
 		       "WEP" : "none",
 
-		       test_bit(Ndis802_11Encryption2Enabled, &wd->capa.encr) ?
+		       test_bit(Ndis802_11Encryption2Enabled,
+				&wnd->capa.encr) ?
 		       "; TKIP with WPA" : "",
-		       test_bit(Ndis802_11AuthModeWPA2, &wd->capa.auth) ?
+		       test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		       ", WPA2" : "",
-		       test_bit(Ndis802_11AuthModeWPA2PSK, &wd->capa.auth) ?
+		       test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
 		       ", WPA2PSK" : "",
 
-		       test_bit(Ndis802_11Encryption3Enabled, &wd->capa.encr) ?
+		       test_bit(Ndis802_11Encryption3Enabled,
+				&wnd->capa.encr) ?
 		       "; AES/CCMP with WPA" : "",
-		       test_bit(Ndis802_11AuthModeWPA2, &wd->capa.auth) ?
+		       test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		       ", WPA2" : "",
-		       test_bit(Ndis802_11AuthModeWPA2PSK, &wd->capa.auth) ?
+		       test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
 		       ", WPA2PSK" : "");
 	} else if (!strcmp(setting, "suspend")) {
 		int i;
@@ -324,26 +329,32 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		i = simple_strtol(p, NULL, 10);
 		if (i <= 0 || i > 3)
 			return -EINVAL;
-		if (wd->ndis_device->bustype == NDIS_PCI_BUS)
-			ndiswrapper_suspend_pci(wd->dev.pci, PMSG_SUSPEND);
-#if defined(CONFIG_USB) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		else if (wd->ndis_device->bustype == NDIS_USB_BUS)
-			ndiswrapper_suspend_usb(wd->dev.usb.intf,
-						PMSG_SUSPEND);
-#endif
+		if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
+			i = wrap_pnp_suspend_pci_device(wnd->wd->pci.pdev,
+							PMSG_SUSPEND);
 		else
+#if defined(CONFIG_USB) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+			i = wrap_pnp_suspend_usb_device(wnd->wd->usb.intf,
+							PMSG_SUSPEND);
+#else
+		i = -1;
+#endif
+		if (i)
 			return -EINVAL;
 	} else if (!strcmp(setting, "resume")) {
-		if (wd->ndis_device->bustype == NDIS_PCI_BUS)
-			ndiswrapper_resume_pci(wd->dev.pci);
-#if defined(CONFIG_USB) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		else if (wd->ndis_device->bustype == NDIS_USB_BUS)
-			ndiswrapper_resume_usb(wd->dev.usb.intf);
-#endif
+		int i;
+		if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
+			i = wrap_pnp_resume_pci_device(wnd->wd->pci.pdev);
 		else
+#if defined(CONFIG_USB) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+			i = wrap_pnp_resume_usb_device(wnd->wd->usb.intf);
+#else
+		i = -1;
+#endif
+		if (i)
 			return -EINVAL;
 	} else if (!strcmp(setting, "reinit")) {
-		if (ndis_reinit(wd))
+		if (ndis_reinit(wnd))
 			return -EINVAL;
 #ifdef USB_DEBUG
 	} else if (!strcmp(setting, "irp")) {
@@ -383,7 +394,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		if (i < 0 || i > 1)
 			return -EINVAL;
 
-		miniport = &wd->driver->miniport;
+		miniport = &wnd->wd->driver->ndis_driver->miniport;
 		if (!miniport->pnp_event_notify)
 			return -EFAULT;
 
@@ -393,7 +404,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		else
 			profile_inf = NdisPowerProfileBattery;
 		
-		miniport->pnp_event_notify(wd->nmb->adapter_ctx,
+		miniport->pnp_event_notify(wnd->nmb->adapter_ctx,
 					   NdisDevicePnPEventPowerProfileChanged,
 					   &profile_inf, sizeof(profile_inf));
 	} else if (!strcmp(setting, "auth_mode")) {
@@ -406,7 +417,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		if (i <= 0 || i > 5)
 			return -EINVAL;
 
-		if (set_auth_mode(wd, i))
+		if (set_auth_mode(wnd, i))
 			return -EINVAL;
 	} else if (!strcmp(setting, "encr_mode")) {
 		int i;
@@ -418,7 +429,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		if (i <= 0 || i > 7)
 			return -EINVAL;
 
-		if (set_encr_mode(wd, i))
+		if (set_encr_mode(wnd, i))
 			return -EINVAL;
 	} else if (!strcmp(setting, "stats_enabled")) {
 		int i;
@@ -428,9 +439,9 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		p++;
 		i = simple_strtol(p, NULL, 10);
 		if (i > 0)
-			wd->stats_enabled = TRUE;
+			wnd->stats_enabled = TRUE;
 		else
-			wd->stats_enabled = FALSE;
+			wnd->stats_enabled = FALSE;
 	} else if (!strcmp(setting, "tx_antenna")) {
 		ndis_antenna antenna;
 		unsigned int i;
@@ -440,13 +451,13 @@ static int procfs_write_settings(struct file *file, const char *buf,
 			return -EINVAL;
 		p++;
 		i = simple_strtol(p, NULL, 16);
-		res = miniport_query_info(wd, OID_802_11_NUMBER_OF_ANTENNAS,
+		res = miniport_query_info(wnd, OID_802_11_NUMBER_OF_ANTENNAS,
 					  &antenna, sizeof(antenna));
 		if (res)
 			return -EINVAL;
 		if (i >= -1 && i < antenna)
 			antenna = i;
-		res = miniport_set_info(wd, OID_802_11_TX_ANTENNA_SELECTED,
+		res = miniport_set_info(wnd, OID_802_11_TX_ANTENNA_SELECTED,
 				  &antenna, sizeof(antenna));
 		if (res)
 			return -EINVAL;
@@ -459,63 +470,32 @@ static int procfs_write_settings(struct file *file, const char *buf,
 			return -EINVAL;
 		p++;
 		i = simple_strtol(p, NULL, 16);
-		res = miniport_query_info(wd, OID_802_11_NUMBER_OF_ANTENNAS,
+		res = miniport_query_info(wnd, OID_802_11_NUMBER_OF_ANTENNAS,
 					  &antenna, sizeof(antenna));
 		if (res)
 			return -EINVAL;
 		if (i >= -1 && i < antenna)
 			antenna = i;
-		res = miniport_set_info(wd, OID_802_11_RX_ANTENNA_SELECTED,
+		res = miniport_set_info(wnd, OID_802_11_RX_ANTENNA_SELECTED,
 				  &antenna, sizeof(antenna));
 		if (res)
 			return -EINVAL;
-	} else {
-		int res = -1;
-		struct device_setting *dev_setting;
-
-		if (!p)
-			TRACEEXIT1(return -EINVAL);
-		p++;
-		DBGTRACE1("name='%s', value='%s'\n", setting, p);
-		list_for_each_entry(dev_setting, &wd->ndis_device->settings,
-				    list) {
-			struct ndis_config_param *param;
-			struct unicode_string *ustring;
-
-			param = &dev_setting->config_param;
-			if (!stricmp(dev_setting->name, setting)) {
-				if (strlen(p) > MAX_NDIS_SETTING_VALUE_LEN)
-					TRACEEXIT1(return -EINVAL);
-				memset(dev_setting->value, 0,
-				       MAX_NDIS_SETTING_VALUE_LEN);
-				memcpy(dev_setting->value, p, strlen(p));
-				ustring = &param->data.ustring;
-				if (param->type == NDIS_CONFIG_PARAM_STRING)
-					RtlFreeUnicodeString(ustring);
-				param->type = NDIS_CONFIG_PARAM_NONE;
-				res = 0;
-			}
-		}
-		if (res)
-			return -EINVAL;
 	}
-
 	return count;
-
 }
 
-int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd)
+int wrap_procfs_add_ndis_device(struct wrap_ndis_device *wnd)
 {
-	struct net_device *dev = wd->net_dev;
+	struct net_device *dev = wnd->net_dev;
 	struct proc_dir_entry *proc_iface, *procfs_entry;
 
-	wd->procfs_iface = NULL;
-	if (ndiswrapper_procfs_entry == NULL)
+	wnd->procfs_iface = NULL;
+	if (wrap_procfs_entry == NULL)
 		return -ENOMEM;
 
-	proc_iface = proc_mkdir(dev->name, ndiswrapper_procfs_entry);
+	proc_iface = proc_mkdir(dev->name, wrap_procfs_entry);
 
-	wd->procfs_iface = proc_iface;
+	wnd->procfs_iface = proc_iface;
 
 	if (proc_iface == NULL) {
 		ERROR("couldn't create proc directory");
@@ -532,8 +512,8 @@ int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd)
 	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
-		procfs_entry->data = wd;
-		procfs_entry->read_proc = procfs_read_hw;
+		procfs_entry->data = wnd;
+		procfs_entry->read_proc = procfs_read_ndis_hw;
 	}
 
 	procfs_entry = create_proc_entry("stats", S_IFREG | S_IRUSR | S_IRGRP,
@@ -544,8 +524,8 @@ int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd)
 	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
-		procfs_entry->data = wd;
-		procfs_entry->read_proc = procfs_read_stats;
+		procfs_entry->data = wnd;
+		procfs_entry->read_proc = procfs_read_ndis_stats;
 	}
 
 	procfs_entry = create_proc_entry("encr", S_IFREG | S_IRUSR | S_IRGRP,
@@ -556,8 +536,8 @@ int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd)
 	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
-		procfs_entry->data = wd;
-		procfs_entry->read_proc = procfs_read_encr;
+		procfs_entry->data = wnd;
+		procfs_entry->read_proc = procfs_read_ndis_encr;
 	}
 
 	procfs_entry = create_proc_entry("settings", S_IFREG |
@@ -569,17 +549,17 @@ int ndiswrapper_procfs_add_iface(struct wrapper_dev *wd)
 	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
-		procfs_entry->data = wd;
-		procfs_entry->read_proc = procfs_read_settings;
-		procfs_entry->write_proc = procfs_write_settings;
+		procfs_entry->data = wnd;
+		procfs_entry->read_proc = procfs_read_ndis_settings;
+		procfs_entry->write_proc = procfs_write_ndis_settings;
 	}
 	return 0;
 }
 
-void ndiswrapper_procfs_remove_iface(struct wrapper_dev *wd)
+void wrap_procfs_remove_ndis_device(struct wrap_ndis_device *wnd)
 {
-	struct net_device *dev = wd->net_dev;
-	struct proc_dir_entry *procfs_iface = wd->procfs_iface;
+	struct net_device *dev = wnd->net_dev;
+	struct proc_dir_entry *procfs_iface = wnd->procfs_iface;
 
 	if (procfs_iface == NULL)
 		return;
@@ -587,9 +567,9 @@ void ndiswrapper_procfs_remove_iface(struct wrapper_dev *wd)
 	remove_proc_entry("stats", procfs_iface);
 	remove_proc_entry("encr", procfs_iface);
 	remove_proc_entry("settings", procfs_iface);
-	if (ndiswrapper_procfs_entry != NULL)
-		remove_proc_entry(dev->name, ndiswrapper_procfs_entry);
-	wd->procfs_iface = NULL;
+	if (wrap_procfs_entry != NULL)
+		remove_proc_entry(dev->name, wrap_procfs_entry);
+	wnd->procfs_iface = NULL;
 }
 
 static int procfs_read_debug(char *page, char **start, off_t off,
@@ -601,9 +581,7 @@ static int procfs_read_debug(char *page, char **start, off_t off,
 		*eof = 1;
 		return 0;
 	}
-
 	p += sprintf(p, "%d\n", debug);
-
 	return (p - page);
 }
 
@@ -634,20 +612,20 @@ static int procfs_write_debug(struct file *file, const char *buf,
 	return count;
 }
 
-int ndiswrapper_procfs_init(void)
+int wrap_procfs_init(void)
 {
 	struct proc_dir_entry *procfs_entry;
 
-	ndiswrapper_procfs_entry = proc_mkdir(DRIVER_NAME, proc_net);
-	if (ndiswrapper_procfs_entry == NULL) {
+	wrap_procfs_entry = proc_mkdir(DRIVER_NAME, proc_net);
+	if (wrap_procfs_entry == NULL) {
 		ERROR("couldn't create procfs directory");
 		return -ENOMEM;
 	}
-	ndiswrapper_procfs_entry->uid = proc_uid;
-	ndiswrapper_procfs_entry->gid = proc_gid;
+	wrap_procfs_entry->uid = proc_uid;
+	wrap_procfs_entry->gid = proc_gid;
 
 	procfs_entry = create_proc_entry("debug", S_IFREG | S_IRUSR | S_IRGRP,
-					 ndiswrapper_procfs_entry);
+					 wrap_procfs_entry);
 	if (procfs_entry == NULL) {
 		ERROR("couldn't create proc entry for 'debug'");
 		return -ENOMEM;
@@ -657,14 +635,13 @@ int ndiswrapper_procfs_init(void)
 		procfs_entry->read_proc  = procfs_read_debug;
 		procfs_entry->write_proc = procfs_write_debug;
 	}
-
 	return 0;
 }
 
-void ndiswrapper_procfs_remove(void)
+void wrap_procfs_remove(void)
 {
-	if (ndiswrapper_procfs_entry == NULL)
+	if (wrap_procfs_entry == NULL)
 		return;
-	remove_proc_entry("debug", ndiswrapper_procfs_entry);
+	remove_proc_entry("debug", wrap_procfs_entry);
 	remove_proc_entry(DRIVER_NAME, proc_net);
 }
