@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2004 Jan Kiszka
+ *  Copyright (C) 2005 Giridhar Pemmasani
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,32 +19,42 @@
 
 #ifdef USB_DEBUG
 static unsigned int urb_id = 0;
-#endif
 
 #define DUMP_WRAP_URB(wrap_urb, dir)					\
-	USBTRACE("urb %p (%d) %s: buf: %p, len: %d, pipe: %u",	\
+	USBTRACE("urb %p (%d) %s: buf: %p, len: %d, pipe: 0x%x",	\
 		 (wrap_urb)->urb, (wrap_urb)->id,			\
-		 (dir == 0) ? "going down" : "coming back",		\
+		 (dir == USB_DIR_OUT) ? "going down" : "coming back",	\
 		 (wrap_urb)->urb->transfer_buffer,			\
 		 (wrap_urb)->urb->transfer_buffer_length,		\
 		 (wrap_urb)->urb->pipe)
 
-#define DUMP_BUFFER(buf, len)						\
+#define DUMP_URB_BUFFER(urb, dir)					\
 	while (debug >= 2) {						\
-		int __i;						\
-		char __msg[100], *__t;					\
-		if (!buf)						\
+		int i;							\
+		char msg[100], *t;					\
+		if (!urb->transfer_buffer)				\
 			break;						\
-		__t = __msg;						\
-		for (__i = 0; __i < len &&				\
-			     __t < &__msg[sizeof(__msg) - 4]; __i++) {	\
-			__t += sprintf(__t, "%02X ",			\
-				       (((UCHAR *)buf)[__i]));		\
-		}							\
-		USBTRACE("%s", __msg);					\
+		if (!((usb_pipein(urb->pipe) && dir == USB_DIR_IN) ||	\
+		      (usb_pipeout(urb->pipe) && dir == USB_DIR_OUT)))	\
+			break;						\
+		t = msg;						\
+		t += sprintf(t, "%d: ", (urb)->actual_length);		\
+		for (i = 0; i < urb->actual_length &&			\
+			     t < &msg[sizeof(msg) - 4]; i++)		\
+			t += sprintf(t, "%02X ",			\
+				     ((char *)urb->transfer_buffer)[i]); \
+		*t = 0;							\
+		USBTRACE("%s", msg);					\
 		break;							\
 	}
 
+#else
+
+#define DUMP_WRAP_URB(wrap_urb, dir) (void)0
+#define DUMP_URB_BUFFER(urb, dir) (void)0
+
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,5)
 #define CUR_ALT_SETTING(intf) (intf)->cur_altsetting
 #else
@@ -54,6 +65,10 @@ static unsigned int urb_id = 0;
 #define USB_CTRL_SET_TIMEOUT 5000
 #endif
 
+#ifndef USB_CTRL_GET_TIMEOUT
+#define USB_CTRL_GET_TIMEOUT 5000
+#endif
+
 static int inline wrap_cancel_urb(struct urb *urb)
 {
 	int ret;
@@ -95,10 +110,9 @@ static void usb_init_urb(struct urb *urb)
 
 extern KSPIN_LOCK irp_cancel_lock;
 
-/* TODO: using a worker instead of tasklet is probably preferable,
- * since we don't know if IoCompleteRequest can be called from atomic
- * context, but with ZyDas driver the worker never gets executed, even
- * though it gets scheduled */
+/* TODO: using a worker instead of tasklet is probably preferable, but
+ * with ZyDas driver the worker never gets executed, even though it
+ * gets scheduled */
 static struct tasklet_struct irp_complete_work;
 static struct nt_list irp_complete_list;
 
@@ -122,14 +136,14 @@ void usb_exit(void)
 	TRACEEXIT1(return);
 }
 
-int usb_init_device(struct wrapper_dev *wd)
+int usb_init_device(struct wrap_device *wd)
 {
-	InitializeListHead(&wd->dev.usb.wrap_urb_list);
-	wd->dev.usb.num_alloc_urbs = 0;
-	return 0;
+	InitializeListHead(&wd->usb.wrap_urb_list);
+	wd->usb.num_alloc_urbs = 0;
+	USBEXIT(return 0);
 }
 
-void usb_exit_device(struct wrapper_dev *wd)
+void usb_exit_device(struct wrap_device *wd)
 {
 	struct nt_list *ent;
 	struct wrap_urb *wrap_urb;
@@ -137,7 +151,7 @@ void usb_exit_device(struct wrapper_dev *wd)
 
 	while (1) {
 		IoAcquireCancelSpinLock(&irql);
-		ent = RemoveHeadList(&wd->dev.usb.wrap_urb_list);
+		ent = RemoveHeadList(&wd->usb.wrap_urb_list);
 		IoReleaseCancelSpinLock(irql);
 		if (!ent)
 			break;
@@ -155,7 +169,9 @@ void usb_exit_device(struct wrapper_dev *wd)
 		usb_free_urb(wrap_urb->urb);
 		kfree(wrap_urb);
 	}
-	wd->dev.usb.num_alloc_urbs = 0;
+	wd->usb.num_alloc_urbs = 0;
+	wd->usb.udev = NULL;
+	wd->usb.intf = NULL;
 	return;
 }
 
@@ -184,6 +200,8 @@ static USBD_STATUS wrap_urb_status(int urb_status)
 		return USBD_STATUS_DEVICE_GONE;
 	case -ENOMEM:
 		return USBD_STATUS_NO_MEMORY;
+	case -EINVAL:
+		return USBD_STATUS_REQUEST_FAILED;
 	default:
 		return USBD_STATUS_NOT_SUPPORTED;
 	}
@@ -212,7 +230,7 @@ static void wrap_free_urb(struct urb *urb)
 {
 	struct irp *irp;
 	struct wrap_urb *wrap_urb;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 
 	USBTRACE("freeing urb: %p", urb);
 	wrap_urb = urb->context;
@@ -225,15 +243,15 @@ static void wrap_free_urb(struct urb *urb)
 	    (wrap_urb->alloc_flags & URB_NO_TRANSFER_DMA_MAP)) {
 		USBTRACE("freeing DMA buffer for URB: %p %p",
 			 urb, urb->transfer_buffer);
-		usb_buffer_free(irp->wd->dev.usb.udev,
+		usb_buffer_free(irp->wd->usb.udev,
 				urb->transfer_buffer_length, 
 				urb->transfer_buffer, urb->transfer_dma);
 	}
 	if (urb->setup_packet)
 		kfree(urb->setup_packet);
-	if (wd->dev.usb.num_alloc_urbs > MAX_ALLOCATED_URBS) {
+	if (wd->usb.num_alloc_urbs > MAX_ALLOCATED_URBS) {
 		RemoveEntryList(&wrap_urb->list);
-		wd->dev.usb.num_alloc_urbs--;
+		wd->usb.num_alloc_urbs--;
 		usb_free_urb(urb);
 		kfree(wrap_urb);
 	} else {
@@ -241,19 +259,18 @@ static void wrap_free_urb(struct urb *urb)
 		wrap_urb->alloc_flags = 0;
 		wrap_urb->irp = NULL;
 	}
-	if (wd->dev.usb.num_alloc_urbs < 0)
-		WARNING("num_allocated_urbs: %d",
-			wd->dev.usb.num_alloc_urbs);
+	if (wd->usb.num_alloc_urbs < 0)
+		WARNING("num_allocated_urbs: %d", wd->usb.num_alloc_urbs);
 	IoReleaseCancelSpinLock(irp->cancel_irql);
 	return;
 }
 
-void wrap_suspend_urbs(struct wrapper_dev *wd)
+void wrap_suspend_urbs(struct wrap_device *wd)
 {
 	/* TODO: do we need to cancel urbs? */
 }
 
-void wrap_resume_urbs(struct wrapper_dev *wd)
+void wrap_resume_urbs(struct wrap_device *wd)
 {
 	/* TODO: do we need to resubmit urbs? */
 }
@@ -264,8 +281,9 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	struct urb *urb;
 	unsigned int alloc_flags;
 	struct wrap_urb *wrap_urb;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 
+	USBENTER("irp: %p", irp);
 	if (current_irql() < DISPATCH_LEVEL)
 		alloc_flags = GFP_KERNEL;
 	else
@@ -273,7 +291,7 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	wd = irp->wd;
 	IoAcquireCancelSpinLock(&irp->cancel_irql);
 	urb = NULL;
-	nt_list_for_each_entry(wrap_urb, &wd->dev.usb.wrap_urb_list, list) {
+	nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list, list) {
 		if (wrap_urb->state == URB_FREE) {
 			wrap_urb->state = URB_ALLOCATED;
 			urb = wrap_urb->urb;
@@ -281,7 +299,6 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 			break;
 		}
 	}
-
 	if (!urb) {
 		IoReleaseCancelSpinLock(irp->cancel_irql);
 		wrap_urb = kmalloc(sizeof(*wrap_urb), alloc_flags);
@@ -299,15 +316,15 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 			kfree(wrap_urb);
 			return NULL;
 		}
-		IoAcquireCancelSpinLock(&irp->cancel_irql);
 		memset(wrap_urb, 0, sizeof(*wrap_urb));
+		IoAcquireCancelSpinLock(&irp->cancel_irql);
 		wrap_urb->urb = urb;
 		wrap_urb->state = URB_ALLOCATED;
-		InsertTailList(&wd->dev.usb.wrap_urb_list, &wrap_urb->list);
-		wd->dev.usb.num_alloc_urbs++;
+		InsertTailList(&wd->usb.wrap_urb_list, &wrap_urb->list);
+		wd->usb.num_alloc_urbs++;
 	}
 
-#ifdef USB_ASYNC_UNLINK
+#ifdef URB_ASYNC_UNLINK
 	urb->transfer_flags |= URB_ASYNC_UNLINK;
 #endif
 	urb->context = wrap_urb;
@@ -319,11 +336,15 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	IoReleaseCancelSpinLock(irp->cancel_irql);
 	USBTRACE("allocated urb: %p", urb);
 	if (buf_len && buf) {
-		if (virt_addr_valid(buf))
+		if (
+#if defined(CONFIG_HIGHMEM) || defined(CONFIG_HIGHMEM4G)
+			0 &&
+#endif
+			virt_addr_valid(buf))
 			urb->transfer_buffer = buf;
 		else {
 			urb->transfer_buffer =
-				usb_buffer_alloc(irp->wd->dev.usb.udev,
+				usb_buffer_alloc(wd->usb.udev,
 						 buf_len, alloc_flags,
 						 &urb->transfer_dma);
 			if (!urb->transfer_buffer) {
@@ -377,10 +398,11 @@ NTSTATUS wrap_submit_urb(struct irp *irp)
 	irp->wrap_urb->id = urb_id++;
 #endif
 	IoReleaseCancelSpinLock(irp->cancel_irql);
-	DUMP_WRAP_URB(irp->wrap_urb, 0);
+	DUMP_WRAP_URB(irp->wrap_urb, USB_DIR_OUT);
 	irp->io_status.status = STATUS_PENDING;
 	irp->io_status.status_info = 0;
 	NT_URB_STATUS(nt_urb) = USBD_STATUS_PENDING;
+	DUMP_URB_BUFFER(urb, USB_DIR_OUT);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	ret = usb_submit_urb(urb, alloc_flags);
 #else
@@ -409,13 +431,13 @@ static void wrap_urb_complete(struct urb *urb)
 	wrap_urb = urb->context;
 	irp = wrap_urb->irp;
 	IoAcquireCancelSpinLock(&irp->cancel_irql);
-	DUMP_WRAP_URB(wrap_urb, 1);
+	DUMP_WRAP_URB(wrap_urb, USB_DIR_IN);
 	irp->cancel_routine = NULL;
 	if (wrap_urb->state != URB_SUBMITTED &&
 	    wrap_urb->state != URB_CANCELED)
 		WARNING("urb %p in wrong state: %d", urb, wrap_urb->state);
 	wrap_urb->state = URB_COMPLETED;
-	/* To prevent 2.4 kernels from resubmiting interrupt URBs
+	/* Prevent 2.4 kernels from resubmiting interrupt URBs
 	 * (Windows driver also resubmits them); UHCI doesn't resubmit
 	 * URB if status == -ENOENT */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
@@ -452,7 +474,8 @@ static void irp_complete_worker(unsigned long data)
 		DUMP_IRP(irp);
 		wrap_urb = irp->wrap_urb;
 		urb = wrap_urb->urb;
-		if (wrap_urb->state != URB_COMPLETED)
+		if (wrap_urb->state != URB_COMPLETED &&
+		    wrap_urb->state != URB_CANCELED)
 			WARNING("urb %p in wrong state: %d",
 				urb, wrap_urb->state);
 		nt_urb = URB_FROM_IRP(irp);
@@ -468,53 +491,34 @@ static void irp_complete_worker(unsigned long data)
 
 			/* from WDM examples, it seems we don't need
 			 * to update MDL's byte count if MDL is used */
-			switch (nt_urb->header.function) {
-			case URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER:
+			if (nt_urb->header.function ==
+			    URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER) {
 				bulk_int_tx = &nt_urb->bulk_int_transfer;
 				bulk_int_tx->transfer_buffer_length =
 					urb->actual_length;
-				DUMP_BUFFER(urb->transfer_buffer,
-					    urb->actual_length);
+				DUMP_URB_BUFFER(urb, USB_DIR_IN);
 				if ((wrap_urb->alloc_flags &
 				     URB_NO_TRANSFER_DMA_MAP) &&
 				    usb_pipein(urb->pipe))
 					memcpy(bulk_int_tx->transfer_buffer,
 					       urb->transfer_buffer,
 					       urb->actual_length);
-				break;
-			case URB_FUNCTION_VENDOR_DEVICE:
-			case URB_FUNCTION_VENDOR_INTERFACE:
-			case URB_FUNCTION_VENDOR_ENDPOINT:
-			case URB_FUNCTION_VENDOR_OTHER:
-			case URB_FUNCTION_CLASS_DEVICE:
-			case URB_FUNCTION_CLASS_INTERFACE:
-			case URB_FUNCTION_CLASS_ENDPOINT:
-			case URB_FUNCTION_CLASS_OTHER:
+			} else {
 				vc_req = &nt_urb->vendor_class_request;
 				vc_req->transfer_buffer_length =
 					urb->actual_length;
-				DUMP_BUFFER(urb->transfer_buffer,
-					    urb->actual_length);
-				DUMP_BUFFER(urb->setup_packet,
-					    sizeof(struct usb_ctrlrequest));
+				DUMP_URB_BUFFER(urb, USB_DIR_IN);
 				if ((wrap_urb->alloc_flags &
 				     URB_NO_TRANSFER_DMA_MAP) &&
 				    usb_pipein(urb->pipe))
 					memcpy(vc_req->transfer_buffer,
 					       urb->transfer_buffer,
 					       urb->actual_length);
-				break;
-			default:
-				ERROR("nt_urb type: %d unknown",
-				      nt_urb->header.function);
-				break;
 			}
 			break;
 		case -ENOENT:
 		case -ECONNRESET:
 			/* irp canceled */
-			if (wrap_urb->state == URB_SUSPEND)
-				continue;
 			NT_URB_STATUS(nt_urb) = USBD_STATUS_CANCELLED;
 			irp->io_status.status = STATUS_CANCELLED;
 			irp->io_status.status_info = 0;
@@ -575,19 +579,29 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 	union nt_urb *nt_urb;
 
 	nt_urb = URB_FROM_IRP(irp);
-	udev = irp->wd->dev.usb.udev;
+	udev = irp->wd->usb.udev;
 	bulk_int_tx = &nt_urb->bulk_int_transfer;
-	USBTRACE("flags = %X, length = %u, buffer = %p",
-		  bulk_int_tx->transfer_flags,
-		  bulk_int_tx->transfer_buffer_length,
-		  bulk_int_tx->transfer_buffer);
-
-	DUMP_IRP(irp);
 	pipe_handle = bulk_int_tx->pipe_handle;
-	if (bulk_int_tx->transfer_flags & USBD_TRANSFER_DIRECTION_IN)
-		pipe = usb_rcvbulkpipe(udev, pipe_handle->bEndpointAddress);
-	else
-		pipe = usb_sndbulkpipe(udev, pipe_handle->bEndpointAddress);
+	USBTRACE("flags = %X, length = %u, buffer = %p, handle: %p",
+		 bulk_int_tx->transfer_flags,
+		 bulk_int_tx->transfer_buffer_length,
+		 bulk_int_tx->transfer_buffer, pipe_handle);
+
+	if (bulk_int_tx->transfer_flags & USBD_TRANSFER_DIRECTION_IN) {
+		if (USBD_IS_BULK_PIPE(pipe_handle))
+			pipe = usb_rcvbulkpipe(udev,
+					       pipe_handle->bEndpointAddress);
+		else
+			pipe = usb_rcvintpipe(udev,
+					      pipe_handle->bEndpointAddress);
+	} else {
+		if (USBD_IS_BULK_PIPE(pipe_handle))
+			pipe = usb_sndbulkpipe(udev,
+					       pipe_handle->bEndpointAddress);
+		else
+			pipe = usb_sndintpipe(udev,
+					      pipe_handle->bEndpointAddress);
+	}
 
 	if (unlikely(bulk_int_tx->transfer_buffer == NULL &&
 		     bulk_int_tx->transfer_buffer_length > 0)) {
@@ -600,6 +614,7 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 			MmGetMdlVirtualAddress(bulk_int_tx->mdl);
 	}
 
+	DUMP_IRP(irp);
 	urb = wrap_alloc_urb(irp, pipe, bulk_int_tx->transfer_buffer,
 			     bulk_int_tx->transfer_buffer_length);
 	if (!urb) {
@@ -611,30 +626,24 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 		USBTRACE("short not ok");
 		urb->transfer_flags |= URB_SHORT_NOT_OK;
 	}
-
-	switch(usb_pipetype(pipe)) {
-	case USB_ENDPOINT_XFER_BULK:
+	
+	if (usb_pipebulk(pipe)) {
 		usb_fill_bulk_urb(urb, udev, pipe, urb->transfer_buffer,
 				  bulk_int_tx->transfer_buffer_length,
 				  wrap_urb_complete, urb->context);
-		USBTRACE("submitting bulk urb %p on pipe %u",
-			 urb, pipe_handle->bEndpointAddress);
+		USBTRACE("submitting bulk urb %p on pipe 0x%x (ep 0x%x)",
+			 urb, urb->pipe, pipe_handle->bEndpointAddress);
 		status = USBD_STATUS_PENDING;
-		break;
-	case USB_ENDPOINT_XFER_INT:
+	} else {
 		usb_fill_int_urb(urb, udev, pipe, urb->transfer_buffer,
 				 bulk_int_tx->transfer_buffer_length,
 				 wrap_urb_complete, urb->context,
 				 pipe_handle->bInterval);
-		USBTRACE("submitting interrupt urb %p on pipe %u, intvl: %d",
-			 urb, pipe_handle->bEndpointAddress,
+		USBTRACE("submitting interrupt urb %p on pipe 0x%x (ep 0x%x), "
+			 "intvl: %d", urb, urb->pipe,
+			 pipe_handle->bEndpointAddress,
 			 pipe_handle->bInterval);
 		status = USBD_STATUS_PENDING;
-		break;
-	default:
-		ERROR("unknown pipe type: %u", pipe_handle->bEndpointAddress);
-		status = USBD_STATUS_NOT_SUPPORTED;
-		break;
 	}
 	USBEXIT(return status);
 }
@@ -651,7 +660,7 @@ static USBD_STATUS wrap_vendor_or_class_req(struct irp *irp)
 	USBD_STATUS status;
 
 	nt_urb = URB_FROM_IRP(irp);
-	udev = irp->wd->dev.usb.udev;
+	udev = irp->wd->usb.udev;
 	vc_req = &nt_urb->vendor_class_request;
 	USBTRACE("bits = %x, req = %x, val = %08x, index = %08x, flags = %x,"
 		 "tx_buf = %p, tx_buf_len = %d", vc_req->reserved_bits,
@@ -758,7 +767,6 @@ static USBD_STATUS wrap_reset_pipe(struct usb_device *udev, struct irp *irp)
 
 	USBTRACE("irp = %p", irp);
 	nt_urb = URB_FROM_IRP(irp);
-
 	pipe_handle = nt_urb->pipe_req.pipe_handle;
 	pipe_type = pipe_handle->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
 	/* TODO: not clear if both directions should be cleared? */
@@ -774,18 +782,20 @@ static USBD_STATUS wrap_reset_pipe(struct usb_device *udev, struct irp *irp)
 		pipe2 = usb_sndctrlpipe(udev, pipe_handle->bEndpointAddress);
 	} else {
 		WARNING("pipe type %d not handled", pipe_type);
-		return USBD_STATUS_SUCCESS;
+		return USBD_STATUS_INVALID_PIPE_HANDLE;
 	}
 	ret = usb_clear_halt(udev, pipe1);
-	if (ret)
+	if (ret) {
 		WARNING("resetting pipe %d failed: %d", pipe_type, ret);
+		return wrap_urb_status(ret);
+	}
 	if (pipe2 != pipe1) {
 		ret = usb_clear_halt(udev, pipe2);
 		if (ret)
 			WARNING("resetting pipe %d failed: %d",
 				pipe_type, ret);
 	}
-	return USBD_STATUS_SUCCESS;
+	return wrap_urb_status(ret);
 }
 
 static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
@@ -795,18 +805,17 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 	unsigned long flags;
 	struct urb *urb;
 	struct wrap_urb *wrap_urb;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 
 	USBENTER("irp = %p", irp);
 	wd = irp->wd;
 	nt_urb = URB_FROM_IRP(irp);
 	pipe_handle = nt_urb->pipe_req.pipe_handle;
-
 	nt_urb = URB_FROM_IRP(irp);
 	while (1) {
 		kspin_lock_irqsave(&irp_cancel_lock, flags);
 		urb = NULL;
-		nt_list_for_each_entry(wrap_urb, &wd->dev.usb.wrap_urb_list,
+		nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list,
 				       list) {
 			if (wrap_urb->state == URB_SUBMITTED &&
 			    (usb_pipeendpoint(wrap_urb->pipe) ==
@@ -823,23 +832,68 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 		} else
 			break;
 	}
-	USBEXIT(return USBD_STATUS_SUCCESS);
+	if (wd->usb.intf == NULL)
+		USBEXIT(return USBD_STATUS_DEVICE_GONE);
+	else
+		USBEXIT(return USBD_STATUS_SUCCESS);
 }
 
-static USBD_STATUS wrap_select_configuration(struct wrapper_dev *wd,
+static void set_intf_pipe_info(struct usb_interface *usb_intf,
+			       struct usbd_interface_information *intf)
+{
+	int i, pipe_num;
+	struct usb_endpoint_descriptor *ep;
+	struct usbd_pipe_information *pipe;
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+	for (i = 0; i < CUR_ALT_SETTING(usb_intf)->desc.bNumEndpoints;
+	     i++, pipe_num++) {
+		ep = &(CUR_ALT_SETTING(usb_intf)->endpoint[i]).desc;
+#else
+	for (i = 0; i < CUR_ALT_SETTING(usb_intf).bNumEndpoints;
+	     i++, pipe_num++) {
+		ep = &((CUR_ALT_SETTING(usb_intf)).endpoint[i]);
+#endif
+		if (i >= intf->bNumEndpoints) {
+			ERROR("intf %p has only %d endpoints, "
+			      "ignoring endpoints above %d",
+			      intf, intf->bNumEndpoints, i);
+			break;
+		}
+		pipe = &intf->pipes[i];
+
+		if (pipe->flags & USBD_PF_CHANGE_MAX_PACKET)
+			WARNING("pkt_sz: %d: %d", pipe->wMaxPacketSize,
+				pipe->max_tx_size);
+		USBTRACE("driver wants max_tx_size to %d",
+			 pipe->max_tx_size);
+
+		pipe->wMaxPacketSize = ep->wMaxPacketSize;
+		pipe->bEndpointAddress = ep->bEndpointAddress;
+		pipe->bInterval = ep->bInterval;
+		pipe->type =
+			ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
+		pipe->handle = ep;
+		USBTRACE("%d: ep 0x%x, type %d, pkt_sz %d, intv %d,"
+			 "type: %d, handle %p", i,
+			 ep->bEndpointAddress, ep->bmAttributes,
+			 ep->wMaxPacketSize, ep->bInterval,
+			 pipe->type, pipe->handle);
+	}
+}
+
+static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 					     union nt_urb *nt_urb,
 					     struct irp *irp)
 {
-	struct usbd_pipe_information *pipe;
-	int i, n, ret, pipe_num;
-	struct usb_endpoint_descriptor *ep;
+	int n, ret;
 	struct usbd_select_configuration *sel_conf;
 	struct usb_device *udev;
 	struct usbd_interface_information *intf;
 	struct usb_config_descriptor *config;
 	struct usb_interface *usb_intf;
 
-	udev = wd->dev.usb.udev;
+	udev = wd->usb.udev;
 	sel_conf = &nt_urb->select_conf;
 	config = sel_conf->config;
 	if (config == NULL) {
@@ -865,19 +919,20 @@ static USBD_STATUS wrap_select_configuration(struct wrapper_dev *wd,
 		return wrap_urb_status(ret);
 	}
 
-	pipe_num = 0;
 	intf = &sel_conf->intf;
 	for (n = 0; n < config->bNumInterfaces && intf->bLength > 0;
 	     n++, intf = (((void *)intf) + intf->bLength)) {
 
 		USBTRACE("intf: %d, alt setting: %d",
 			 intf->bInterfaceNumber, intf->bAlternateSetting);
+#if 0
 		ret = usb_set_interface(udev, intf->bInterfaceNumber,
 					intf->bAlternateSetting);
 		if (ret < 0) {
 			ERROR("failed with %d", ret);
 			return wrap_urb_status(ret);
 		}
+#endif
 		usb_intf = usb_ifnum_to_if(udev, intf->bInterfaceNumber);
 		if (!usb_intf) {
 			ERROR("couldn't obtain ifnum");
@@ -885,53 +940,68 @@ static USBD_STATUS wrap_select_configuration(struct wrapper_dev *wd,
 		}
 		USBTRACE("intf: %p, num ep: %d", intf, intf->bNumEndpoints);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		for (i = 0; i < CUR_ALT_SETTING(usb_intf)->desc.bNumEndpoints;
-		     i++, pipe_num++) {
-			ep = &(CUR_ALT_SETTING(usb_intf)->endpoint + i)->desc;
-#else
-		for (i = 0; i < CUR_ALT_SETTING(usb_intf).bNumEndpoints;
-		     i++, pipe_num++) {
-			ep = &((CUR_ALT_SETTING(usb_intf)).endpoint[i]);
-#endif
-			if (i >= intf->bNumEndpoints) {
-				ERROR("intf %p has only %d endpoints, "
-				      "ignoring endpoints above %d",
-				      intf, intf->bNumEndpoints, i);
-				break;
-			}
-			pipe = &intf->pipes[i];
-
-			if (pipe->flags & USBD_PF_CHANGE_MAX_PACKET)
-				WARNING("pkt_sz: %d: %d", pipe->wMaxPacketSize,
-					pipe->max_tx_size);
-			USBTRACE("driver wants max_tx_size to %d",
-				 pipe->max_tx_size);
-
-			pipe->wMaxPacketSize = ep->wMaxPacketSize;
-			pipe->bEndpointAddress = ep->bEndpointAddress;
-			pipe->bInterval = ep->bInterval;
-			pipe->type =
-				ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK;
-
-			pipe->handle = ep;
-			USBTRACE("%d: addr %X, type %d, pkt_sz %d, intv %d,"
-				 "handle %p", i, ep->bEndpointAddress,
-				 ep->bmAttributes, ep->wMaxPacketSize,
-				 ep->bInterval, pipe->handle);
-		}
+		set_intf_pipe_info(usb_intf, intf);
 	}
 	return USBD_STATUS_SUCCESS;
 }
 
-static USBD_STATUS wrap_get_descriptor(struct wrapper_dev *wd,
+static USBD_STATUS wrap_select_interface(struct wrap_device *wd,
+					 union nt_urb *nt_urb,
+					 struct irp *irp)
+{
+	int ret;
+	struct usbd_select_interface *sel_intf;
+	struct usb_device *udev;
+	struct usbd_interface_information *intf;
+	struct usb_interface *usb_intf;
+
+	udev = wd->usb.udev;
+	sel_intf = &nt_urb->select_intf;
+	intf = &sel_intf->intf;
+
+	ret = usb_set_interface(udev, intf->bInterfaceNumber,
+				intf->bAlternateSetting);
+	if (ret < 0) {
+		ERROR("failed with %d", ret);
+		return wrap_urb_status(ret);
+	}
+	usb_intf = usb_ifnum_to_if(udev, intf->bInterfaceNumber);
+	if (!usb_intf) {
+		ERROR("couldn't get interface information");
+		return USBD_STATUS_REQUEST_FAILED;
+	}
+	wd->usb.intf = usb_intf;
+	USBTRACE("intf: %p, num ep: %d", intf, intf->bNumEndpoints);
+
+	set_intf_pipe_info(usb_intf, intf);
+	return USBD_STATUS_SUCCESS;
+}
+
+static int wrap_usb_get_string(struct usb_device *udev, unsigned short langid,
+			       unsigned char index, void *buf, int size)
+{
+	int i, ret;
+	/* if langid is 0, return array of langauges supported in
+	 * buf */
+	for (i = 0; i < 3; i++) {
+		ret = usb_control_msg(udev, usb_rcvctrlpipe(udev, 0),
+				      USB_REQ_GET_DESCRIPTOR, USB_DIR_IN,
+				      (USB_DT_STRING << 8) + index, langid,
+				      buf, size, USB_CTRL_GET_TIMEOUT);
+		if (ret > 0 || ret == -EPIPE)
+			break;
+	}
+	return ret;
+}
+
+static USBD_STATUS wrap_get_descriptor(struct wrap_device *wd,
 				       union nt_urb *nt_urb, struct irp *irp)
 {
 	struct usbd_control_descriptor_request *ctrl_req;
 	int ret = 0;
 	struct usb_device *udev;
 
-	udev = wd->dev.usb.udev;
+	udev = wd->usb.udev;
 	ctrl_req = &nt_urb->control_request;
 	USBTRACE("desctype = %d, descindex = %d, transfer_buffer = %p,"
 		 "transfer_buffer_length = %d", ctrl_req->desc_type,
@@ -939,11 +1009,11 @@ static USBD_STATUS wrap_get_descriptor(struct wrapper_dev *wd,
 		 ctrl_req->transfer_buffer_length);
 
 	if (ctrl_req->desc_type == USB_DT_STRING) {
-		USBTRACE("langid: %d", ctrl_req->language_id);
-		ret = usb_get_string(udev, ctrl_req->language_id,
-				     ctrl_req->index,
-				     ctrl_req->transfer_buffer,
-				     ctrl_req->transfer_buffer_length);
+		USBTRACE("langid: %x", ctrl_req->language_id);
+		ret = wrap_usb_get_string(udev, ctrl_req->language_id,
+					  ctrl_req->index,
+					  ctrl_req->transfer_buffer,
+					  ctrl_req->transfer_buffer_length);
 	} else {
 		ret = usb_get_descriptor(udev, ctrl_req->desc_type,
 					 ctrl_req->index,
@@ -953,10 +1023,9 @@ static USBD_STATUS wrap_get_descriptor(struct wrapper_dev *wd,
 	if (ret < 0) {
 		WARNING("request %d failed with %d", ctrl_req->desc_type, ret);
 		ctrl_req->transfer_buffer_length = 0;
-		return USBD_STATUS_REQUEST_FAILED;
+		return wrap_urb_status(ret);
 	} else {
 		USBTRACE("ret: %08x", ret);
-		DUMP_BUFFER(ctrl_req->transfer_buffer, ret);
 		ctrl_req->transfer_buffer_length = ret;
 		irp->io_status.status_info = ret;
 		return USBD_STATUS_SUCCESS;
@@ -968,10 +1037,10 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 	union nt_urb *nt_urb;
 	struct usb_device *udev;
 	USBD_STATUS status;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 
 	wd = irp->wd;
-	udev = wd->dev.usb.udev;
+	udev = wd->usb.udev;
 	nt_urb = URB_FROM_IRP(irp);
 	USBENTER("nt_urb = %p, irp = %p, length = %d, function = %x",
 		 nt_urb, irp, nt_urb->header.length, nt_urb->header.function);
@@ -1003,6 +1072,11 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 		NT_URB_STATUS(nt_urb) = status;
 		break;
 
+	case URB_FUNCTION_SELECT_INTERFACE:
+		status = wrap_select_interface(wd, nt_urb, irp);
+		NT_URB_STATUS(nt_urb) = status;
+		break;
+
 	case URB_FUNCTION_GET_DESCRIPTOR_FROM_DEVICE:
 		status = wrap_get_descriptor(wd, nt_urb, irp);
 		NT_URB_STATUS(nt_urb) = status;
@@ -1030,35 +1104,35 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 static USBD_STATUS wrap_reset_port(struct irp *irp)
 {
 	int ret;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 
 	wd = irp->wd;
-	USBENTER("%p, %p", wd, wd->dev.usb.udev);
-
-	ret = usb_reset_device(wd->dev.usb.udev);
+	USBENTER("%p, %p", wd, wd->usb.udev);
+	ret = usb_reset_device(wd->usb.udev);
 	if (ret < 0)
 		WARNING("reset failed: %d", ret);
-	return USBD_STATUS_SUCCESS;
+	return wrap_urb_status(ret);
 }
 
 static USBD_STATUS wrap_get_port_status(struct irp *irp)
 {
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 	ULONG *status;
 
 	wd = irp->wd;
-	USBENTER("%p, %p", wd, wd->dev.usb.udev);
+	USBENTER("%p, %p", wd, wd->usb.udev);
 	status = IoGetCurrentIrpStackLocation(irp)->params.others.arg1;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	{
 		enum usb_device_state state;
-		state = wd->dev.usb.udev->state;
+		state = wd->usb.udev->state;
 		if (state != USB_STATE_NOTATTACHED &&
 		    state != USB_STATE_SUSPENDED) {
 			*status |= USBD_PORT_CONNECTED;
 			if (state == USB_STATE_CONFIGURED)
 				*status |= USBD_PORT_ENABLED;
 		}
+		USBTRACE("state: %d, *status: %08X", state, *status);
 	}
 #else
 	{
@@ -1072,19 +1146,21 @@ static USBD_STATUS wrap_get_port_status(struct irp *irp)
 NTSTATUS wrap_submit_irp(struct device_object *pdo, struct irp *irp)
 {
 	struct io_stack_location *irp_sl;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 	USBD_STATUS status;
+	struct usbd_idle_callback *idle_callback;
+	union nt_urb *nt_urb;
 
 	irp_sl = IoGetCurrentIrpStackLocation(irp);
 	wd = pdo->reserved;
-
-	if (unlikely(wd->dev.usb.intf == NULL)) {
+	irp->wd = wd;
+	if (unlikely(wd->usb.intf == NULL)) {
+		nt_urb = URB_FROM_IRP(irp);
+		status = NT_URB_STATUS(nt_urb) = USBD_STATUS_DEVICE_GONE;
 		irp->io_status.status = STATUS_DEVICE_REMOVED;
-		irp->io_status.status_info = 0;
-		return irp->io_status.status;
+		USBEXIT(return status);
 	}
 
-	irp->wd = wd;
 	switch (irp_sl->params.ioctl.code) {
 	case IOCTL_INTERNAL_USB_SUBMIT_URB:
 		status = wrap_process_nt_urb(irp);
@@ -1095,6 +1171,11 @@ NTSTATUS wrap_submit_irp(struct device_object *pdo, struct irp *irp)
 	case IOCTL_INTERNAL_USB_GET_PORT_STATUS:
 		status = wrap_get_port_status(irp);
 		break;
+	case IOCTL_INTERNAL_USB_SUBMIT_IDLE_NOTIFICATION:
+		idle_callback = irp_sl->params.ioctl.type3_input_buf;
+		USBTRACE("suspend function: %p", idle_callback->callback);
+		status = USBD_STATUS_NOT_SUPPORTED;
+		break;
 	default:
  		ERROR("ioctl %08X NOT IMPLEMENTED", irp_sl->params.ioctl.code);
 		status = USBD_STATUS_NOT_SUPPORTED;
@@ -1253,4 +1334,102 @@ WRAP_EXPORT(USBD_ParseConfigurationDescriptor)
 						   -1, -1, -1);
 }
 
+STDCALL void WRAP_EXPORT(USBD_GetUSBDIVersion)
+	(struct usbd_version_info *version_info)
+{
+	/* this function is obsolete in Windows XP */
+	if (version_info) {
+		version_info->usbdi_version = USBDI_VERSION;
+		/* TODO: how do we get this correctly? */
+		version_info->supported_usb_version = 0x110; // 0x200
+	}
+	USBEXIT(return);
+}
+
+STDCALL void
+USBD_InterfaceGetUSBDIVersion(void *context,
+			      struct usbd_version_info *version_info,
+			      ULONG *hcd_capa)
+{
+	struct wrap_device *wd = context;
+
+	if (version_info) {
+		version_info->usbdi_version = USBDI_VERSION;
+		if (wd->usb.udev->speed == USB_SPEED_HIGH)
+			version_info->supported_usb_version = 0x200;
+		else
+			version_info->supported_usb_version = 0x110;
+	}
+	*hcd_capa = USB_HCD_CAPS_SUPPORTS_RT_THREADS;
+	USBEXIT(return);
+}
+
+STDCALL BOOLEAN USBD_InterfaceIsDeviceHighSpeed(void *context)
+{
+	struct wrap_device *wd = context;
+
+	USBTRACE("wd: %p", wd);
+	if (wd->usb.udev->speed == USB_SPEED_HIGH)
+		USBEXIT(return TRUE);
+	else
+		USBEXIT(return FALSE);
+}
+
+STDCALL void USBD_InterfaceReference(void *context)
+{
+	USBTRACE("%p", context);
+	UNIMPL();
+}
+
+STDCALL void USBD_InterfaceDereference(void *context)
+{
+	USBTRACE("%p", context);
+	UNIMPL();
+}
+
+STDCALL NTSTATUS USBD_InterfaceQueryBusTime(void *context, ULONG *frame)
+{
+	struct wrap_device *wd = context;
+
+	*frame = usb_get_current_frame_number(wd->usb.udev);
+	USBEXIT(return STATUS_SUCCESS);
+}
+
+STDCALL NTSTATUS USBD_InterfaceSubmitIsoOutUrb(void *context,
+					       union nt_urb *nt_urb)
+{
+	/* TODO: implement this */
+	UNIMPL();
+	USBEXIT(return STATUS_NOT_IMPLEMENTED);
+}
+
+STDCALL NTSTATUS
+USBD_InterfaceQueryBusInformation(void *context, ULONG level, void *buf,
+				  ULONG *buf_length, ULONG *buf_actual_length)
+{
+	struct wrap_device *wd = context;
+	struct usb_bus_information_level *bus_info;
+	struct usb_bus *bus;
+
+	bus = wd->usb.udev->bus;
+	bus_info = buf;
+#if 0
+	/* TODO: implement this */
+	bus_info->TotalBandwidth = 0;
+	bus_info->ConsumedBandwidth =
+		bus->bandwidth_allocated / 1000000;
+#else
+	UNIMPL();
+#endif
+	USBEXIT(return STATUS_NOT_IMPLEMENTED);
+}
+
+STDCALL NTSTATUS
+USBD_InterfaceLogEntry(void *context, ULONG driver_tag, ULONG enum_tag,
+		       ULONG p1, ULONG p2)
+{
+	ERROR("%p, %x, %x, %x, %x", context, driver_tag, enum_tag, p1, p2);
+	USBEXIT(return STATUS_SUCCESS);
+}
+
 #include "usb_exports.h"
