@@ -13,8 +13,8 @@
  *
  */
 
-#ifndef WINNT_TYPES_H
-#define WINNT_TYPES_H
+#ifndef _WINNT_TYPES_H_
+#define _WINNT_TYPES_H_
 
 #define TRUE				1
 #define FALSE				0
@@ -57,6 +57,7 @@
 #define IRP_MJ_DEVICE_CONTROL		0x0E
 #define IRP_MJ_INTERNAL_DEVICE_CONTROL	0x0F
 #define IRP_MJ_POWER			0x16
+#define IRP_MJ_SYSTEM_CONTROL		0x0E
 #define IRP_MJ_PNP			0x1b
 #define IRP_MJ_MAXIMUM_FUNCTION		0x1b
 
@@ -75,6 +76,8 @@
 #define IRP_MN_STOP_DEVICE		0x04
 #define IRP_MN_QUERY_STOP_DEVICE	0x05
 #define IRP_MN_CANCEL_STOP_DEVICE	0x06
+#define IRP_MN_QUERY_DEVICE_RELATIONS	0x07
+#define IRP_MN_QUERY_INTERFACE		0x08
 
 #define IRP_BUFFERED_IO			0x00000010
 #define IRP_DEALLOCATE_BUFFER		0x00000020
@@ -461,23 +464,23 @@ struct driver_object {
 	CSHORT size;
 	struct device_object *dev_obj;
 	ULONG flags;
-	void *driver_start;
+	void *start;
 	ULONG driver_size;
-	void *driver_section;
+	void *section;
 	struct driver_extension *drv_ext;
-	struct unicode_string driver_name;
+	struct unicode_string name;
 	struct unicode_string *hardware_database;
 	void *fast_io_dispatch;
-	void *driver_init;
-	void *driver_start_io;
-	void (*driver_unload)(struct driver_object *driver) STDCALL;
+	void *init;
+	void *start_io;
+	void (*unload)(struct driver_object *driver) STDCALL;
 	driver_dispatch_t *major_func[IRP_MJ_MAXIMUM_FUNCTION + 1];
 };
 
 struct driver_extension {
 	struct driver_object *drv_obj;
-	NTSTATUS (*add_device_func)(struct driver_object *drv_obj,
-				    struct device_object *dev_obj) STDCALL;
+	NTSTATUS (*add_device)(struct driver_object *drv_obj,
+			       struct device_object *dev_obj) STDCALL;
 	ULONG count;
 	struct unicode_string service_key_name;
 	struct nt_list custom_ext;
@@ -553,6 +556,142 @@ enum power_action {
 	PowerActionWarmEject,
 };
 
+struct guid {
+	ULONG data1;
+	USHORT data2;
+	USHORT data3;
+	UCHAR data4[8];
+};
+
+struct nt_interface {
+	USHORT size;
+	USHORT version;
+	void *context;
+	void (*reference)(void *context) STDCALL;
+	void (*dereference)(void *context) STDCALL;
+};
+
+enum interface_type {
+	InterfaceTypeUndefined = -1, Internal, Isa, Eisa, MicroChannel,
+	TurboChannel, PCIBus, VMEBus, NuBus, PCMCIABus, CBus, MPIBus,
+	MPSABus, ProcessorInternal, InternalPowerBus, PNPISABus,
+	PNPBus, MaximumInterfaceType,
+};
+
+#define CmResourceTypeNull		0
+#define CmResourceTypePort		1
+#define CmResourceTypeInterrupt		2
+#define CmResourceTypeMemory		3
+#define CmResourceTypeDma		4
+#define CmResourceTypeDeviceSpecific	5
+#define CmResourceTypeBusNumber		6
+#define CmResourceTypeMaximum		7
+
+#define CmResourceTypeNonArbitrated	128
+#define CmResourceTypeConfigData	128
+#define CmResourceTypeDevicePrivate	129
+#define CmResourceTypePcCardConfig	130
+#define CmResourceTypeMfCardConfig	131
+
+enum cm_share_disposition {
+	CmResourceShareUndetermined = 0, CmResourceShareDeviceExclusive,
+	CmResourceShareDriverExclusive, CmResourceShareShared
+};
+
+#define CM_RESOURCE_INTERRUPT_LEVEL_SENSITIVE	0
+#define CM_RESOURCE_INTERRUPT_LATCHED		1
+#define CM_RESOURCE_MEMORY_READ_WRITE		0x0000
+#define CM_RESOURCE_MEMORY_READ_ONLY		0x0001
+#define CM_RESOURCE_MEMORY_WRITE_ONLY		0x0002
+#define CM_RESOURCE_MEMORY_PREFETCHABLE		0x0004
+
+#define CM_RESOURCE_MEMORY_COMBINEDWRITE	0x0008
+#define CM_RESOURCE_MEMORY_24			0x0010
+#define CM_RESOURCE_MEMORY_CACHEABLE		0x0020
+
+#define CM_RESOURCE_PORT_MEMORY			0x0000
+#define CM_RESOURCE_PORT_IO			0x0001
+#define CM_RESOURCE_PORT_10_BIT_DECODE		0x0004
+#define CM_RESOURCE_PORT_12_BIT_DECODE		0x0008
+#define CM_RESOURCE_PORT_16_BIT_DECODE		0x0010
+#define CM_RESOURCE_PORT_POSITIVE_DECODE	0x0020
+#define CM_RESOURCE_PORT_PASSIVE_DECODE		0x0040
+#define CM_RESOURCE_PORT_WINDOW_DECODE		0x0080
+
+#define CM_RESOURCE_DMA_8			0x0000
+#define CM_RESOURCE_DMA_16			0x0001
+#define CM_RESOURCE_DMA_32			0x0002
+#define CM_RESOURCE_DMA_8_AND_16		0x0004
+#define CM_RESOURCE_DMA_BUS_MASTER		0x0008
+#define CM_RESOURCE_DMA_TYPE_A			0x0010
+#define CM_RESOURCE_DMA_TYPE_B			0x0020
+#define CM_RESOURCE_DMA_TYPE_F			0x0040
+
+#define MAX_RESOURCES 20
+
+#pragma pack(push,4)
+struct cm_partial_resource_descriptor {
+	UCHAR type;
+	UCHAR share;
+	USHORT flags;
+	union {
+		struct {
+			PHYSICAL_ADDRESS start;
+			ULONG length;
+		} generic;
+		struct {
+			PHYSICAL_ADDRESS start;
+			ULONG length;
+		} port;
+		struct {
+			ULONG level;
+			ULONG vector;
+			KAFFINITY affinity;
+		} interrupt;
+		struct {
+			PHYSICAL_ADDRESS start;
+			ULONG length;
+		} memory;
+		struct {
+			ULONG channel;
+			ULONG port;
+			ULONG reserved1;
+		} dma;
+		struct {
+			ULONG data[3];
+		} device_private;
+		struct {
+			ULONG start;
+			ULONG length;
+			ULONG reserved;
+		} bus_number;
+		struct {
+			ULONG data_size;
+			ULONG reserved1;
+			ULONG reserved2;
+		} device_specific_data;
+	} u;
+};
+#pragma pack(pop)
+
+struct cm_partial_resource_list {
+	USHORT version;
+	USHORT revision;
+	ULONG count;
+	struct cm_partial_resource_descriptor partial_descriptors[1];
+};
+
+struct cm_full_resource_descriptor {
+	enum interface_type interface_type;
+	ULONG bus_number;
+	struct cm_partial_resource_list partial_resource_list;
+};
+
+struct cm_resource_list {
+	ULONG count;
+	struct cm_full_resource_descriptor list[1];
+};
+
 #ifndef CONFIG_X86_64
 #pragma pack(push,4)
 #endif
@@ -580,12 +719,23 @@ struct io_stack_location {
 			ULONG POINTER_ALIGNMENT key;
 			LARGE_INTEGER byte_offset;
 		} write;
+		struct {
+			const struct guid *type;
+			USHORT size;
+			USHORT version;
+			struct nt_interface *intf;
+			void *intf_data;
+		} query_intf;
 		struct {
 			ULONG sys_context;
 			enum power_state_type POINTER_ALIGNMENT type;
 			union power_state POINTER_ALIGNMENT state;
 			enum power_action POINTER_ALIGNMENT shutdown_type;
 		} power;
+		struct {
+			struct cm_resource_list *allocated_resources;
+			struct cm_resource_list *allocated_resources_translated;
+		} start_device;
 		struct {
 			ULONG output_buf_len;
 			ULONG POINTER_ALIGNMENT input_buf_len;
@@ -656,7 +806,6 @@ struct wrap_urb {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 	typeof(((struct urb *)0)->status) urb_status;
 #endif
-
 };
 
 struct irp {
@@ -723,7 +872,7 @@ struct irp {
 	/* ndiswrapper extension */
 	struct wrap_urb *wrap_urb;
 	struct nt_list complete_list;
-	struct wrapper_dev *wd;
+	struct wrap_device *wd;
 };
 
 #define IoSizeOfIrp(stack_size)						\
@@ -748,28 +897,31 @@ do {								\
 	IoGetCurrentIrpStackLocation(IRP)++;			\
 } while (0)
 
-#define IoCopyCurrentIrpStackLocationToNext(IRP) 			\
-do {									\
-	struct io_stack_location *_irp_sl, *_next;			\
-	_irp_sl = IoGetCurrentIrpStackLocation(IRP);			\
-	_next = IoGetNextIrpStackLocation(IRP);				\
-	RtlCopyMemory(_next, _irp_sl,					\
-		      offsetof(struct io_stack_location,		\
-			       completion_routine));			\
-	_next->control = 0;						\
-} while (0)
+static inline void
+IoCopyCurrentIrpStackLocationToNext(struct irp *irp)
+{
+	struct io_stack_location *next;
+	next = IoGetNextIrpStackLocation(irp);
+	memcpy(next, IoGetCurrentIrpStackLocation(irp),
+	       offsetof(struct io_stack_location, completion_routine));
+	next->control = 0;
+}
 
-#define IoSetCompletionRoutine(IRP, ROUTINE, CONTEXT, SUC, ERR, CANCEL) \
-do {								\
-	struct io_stack_location *__irp_sl;				\
-	__irp_sl = IoGetNextIrpStackLocation(IRP);			\
-	__irp_sl->completion_routine = (ROUTINE);			\
-	__irp_sl->context = (CONTEXT);					\
-	__irp_sl->control = 0;						\
-	if (SUC) { __irp_sl->control |= CALL_ON_SUCCESS; }		\
-	if (ERR) { __irp_sl->control |= CALL_ON_ERROR; }		\
-	if (CANCEL) { __irp_sl->control |= CALL_ON_CANCEL; }		\
-} while (0)
+static inline void
+IoSetCompletionRoutine(struct irp *irp, void *routine, void *context,
+		       BOOLEAN success, BOOLEAN error, BOOLEAN cancel)
+{
+	struct io_stack_location *irp_sl = IoGetNextIrpStackLocation(irp);
+	irp_sl->completion_routine = routine;
+	irp_sl->context = context;
+	irp_sl->control = 0;
+	if (success)
+		irp_sl->control |= CALL_ON_SUCCESS;
+	if (error)
+		irp_sl->control |= CALL_ON_ERROR;
+	if (cancel)
+		irp_sl->control |= CALL_ON_CANCEL;
+}
 
 #define IoMarkIrpPending(irp)						\
 	(IoGetCurrentIrpStackLocation((irp))->control |= SL_PENDING_RETURNED)
@@ -778,13 +930,6 @@ do {								\
 #define IRP_DRIVER_CONTEXT(irp) (irp)->tail.overlay.driver_context
 #define IoIrpThread(irp) ((irp)->tail.overlay.thread)
 
-struct guid {
-	unsigned long data1;
-	unsigned short data2;
-	unsigned short data3;
-	unsigned char data4[8];
-};
-
 struct wmi_guid_reg_info {
 	struct guid *guid;
 	ULONG instance_count;
@@ -808,6 +953,7 @@ enum key_value_information_class {
 	KeyValuePartialInformationAlign64
 };
 
+
 struct object_attr {
 	ULONG length;
 	void *root_dir;
@@ -825,6 +971,14 @@ struct file_name_info {
 	wchar_t *name;
 };
 
+struct file_std_info {
+	LARGE_INTEGER alloc_size;
+	LARGE_INTEGER eof;
+	ULONG num_links;
+	BOOLEAN delete_pending;
+	BOOLEAN dir;
+};
+
 enum file_info_class {
 	FileAlignmentInformation = 17,
 	FileAttributeTagInformation = 35,
@@ -849,7 +1003,7 @@ struct common_object_header {
 	struct nt_list list;
 	enum common_object_type type;
 	int size;
-	struct unicode_string *name;
+	char *name;
 	unsigned int ref_count;
 	BOOLEAN close_in_process;
 	BOOLEAN permanent;
@@ -870,11 +1024,12 @@ enum work_queue_type {
 	MaximumWorkQueue
 };
 
+typedef STDCALL void (*WRAP_WORK_FUNC)(void *arg1, void *arg2);
+
 struct io_workitem {
 	enum work_queue_type type;
 	struct device_object *dev_obj;
-	void (*worker_routine)(struct device_object *dev_obj,
-			       void *context) STDCALL;
+	WRAP_WORK_FUNC worker_routine;
 	void *context;
 };
 
@@ -1131,6 +1286,23 @@ struct rtl_query_registry_table {
 	ULONG def_length;
 };
 
+struct io_remove_lock {
+	BOOLEAN removed;
+	BOOLEAN reserved[3];
+	LONG io_count;
+	struct nt_event remove_event;
+};
+
+enum device_relation_type {
+	BusRelations,
+	EjectionRelations,
+	PowerRelations,
+	RemovalRelations,
+	TargetDeviceRelation,
+	SingleBusRelations
+};
+
+
 /* some of the functions below are slightly different from DDK's
  * implementation; e.g., Insert functions return appropriate
  * pointer */
